--[[ File: Friend List Lua file
     borrowed lots of code from Asher (Warsong) and his original KOS List .)    			 
     ////////////////////////////////////////////////////////////////////
     18-Mar-05  code hijacked and messed up for FriendList - alpha release
     xoxo Zmrzlina

]]

Friendlist = {
	initialized = nil;
	settingsLoaded = nil;
	playerName = nil;
	realmName = nil;
	originalName = nil; -- Compatibility.
	onlineBufferList = {};
};


--[[

Friendlist:updateEntry(name,type)

-- Update entry for <name>. <type> determines how to update.

]]--
function Friendlist:updateEntry(name, type)
	if type == "flagRSP" then
		if ( FlagHandler:getFlag("Name", name) ~= nil and FlagHandler:getFlag("Name", name) ~= name ) then
			self:SetAltName(name, FlagHandler:getFlag("Name", name));
		end
		self:SetSurname(name,TooltipHandler:getSurname(name));
		self:SetTitle(name,TooltipHandler:getTitle(name));
	elseif type == "unitID" then
		local unitName = UnitName(name);
		self:SetClass(unitName, UnitClass(name));
		self:SetGuild(unitName, GetGuildInfo(name));
		self:SetLevel(unitName, UnitLevel(name));
		self:SetRace(unitName, UnitRace(name));
		if ( UnitPVPName(name) ~= nil and unitName ~= nil and UnitIsPlayer(name) ~= nil ) then
			self:SetRank(unitName, string.sub(UnitPVPName(name), 1, string.find(UnitPVPName(name), unitName)-1));
		end
	elseif type == "flist" then
		for i=1, GetNumFriends() do
			local name, level, class, area, connected = GetFriendInfo(i);

			if level ~= 0 then
				--FlagRSP:printDebug(level);
				--FlagRSP:printDebug(class);

				self:SetLevel(name, level);
				self:SetClass(name, class);
			end
		end
	end
end




--[[

Friendlist:GetFriendstateInfo(friendstate)

- Get info (r,g,b,text) for friend status friendstate.

]]--
function Friendlist:GetFriendstateInfo(friendstate)
	local r, g, b, text;

	text = FRIENDLIST_LOCALE_FRIENDSTATE_TEXT[friendstate];
	r=1; g=1; b=1;

	if friendstate == -10 then
		r=1; g=0; b=0;
	elseif friendstate == 10 then
		r=0; g=1; b=0;
	elseif friendstate == 0 then
		r=1; g=1; b=1;
	else
		r=1; g=1; b=1;
	end

	if text == nil then text = ""; end

	return r, g, b, text;
end


--[[

Friendlist:GetFriendstateGuildInfo(friendstate)

- Get info (r,g,b,text) for guild friend status friendstate.

]]--
function Friendlist:GetFriendstateGuildInfo(friendstate)
	local r, g, b, text;

	text = FRIENDLIST_LOCALE_GUILDFRIENDSTATE_TEXT[friendstate];
	r=1; g=1; b=1;

	if friendstate == -10 then
		r=1; g=0.33; b=0.33;
	elseif friendstate == 10 then
		r=0.33; g=1; b=0.33;
	end

	return r, g, b, text;
end


--[[

Friendlist:GetFriendstateText(name)

- Get player names friend status text.

]]--
function Friendlist:GetFriendstateText(name)
	local friendstate = self:GetFriendstate(name);
	local text = "";

	if friendstate ~= nil then
		--FlagRSP:printDebug("Name is: " .. name);
		--FlagRSP:printDebug("FS is: " .. friendstate);

		if self:GetType(name) ~= "guild" then
			local r, g, b, t = self:GetFriendstateInfo(friendstate);
			if t ~= nil and t ~= "" then
				--text = "|cFF" .. FlagRSP:getHexString(r*255,2) .. FlagRSP:getHexString(g*255,2) .. FlagRSP:getHexString(b*255,2) .. t .. "|r";
				text = t;
			end
		else
			local r, g, b, t = self:GetFriendstateGuildInfo(friendstate);
			if t ~= nil and t ~= "" then
				--text = "|cFF" .. FlagRSP:getHexString(r*255,2) .. FlagRSP:getHexString(g*255,2) .. FlagRSP:getHexString(b*255,2) .. t .. "|r";
				text = t;
			end
		end
	end

	return text;
end


--[[

Friendlist:GetFriendstateColour(name)

- Get player names friend status text colour code.

]]--
function Friendlist:GetFriendstateColour(name)
	local friendstate = self:GetFriendstate(name);
	local text = "";

	if friendstate ~= nil then
		if self:GetType(name) ~= "guild" then
			local r, g, b, t = self:GetFriendstateInfo(friendstate);
			--if t ~= nil and t ~= "" then
			text = FlagRSP:getHexString(r*255,2) .. FlagRSP:getHexString(g*255,2) .. FlagRSP:getHexString(b*255,2);
			--text = t;
			--end
		else
			local r,g,b,t = self:GetFriendstateGuildInfo(friendstate);
			--if t ~= nil and t ~= "" then
			text = FlagRSP:getHexString(r*255,2) .. FlagRSP:getHexString(g*255,2) .. FlagRSP:getHexString(b*255,2);
			--text = t;
			--end
		end
	end

	return text;
end


--[[

Friendlist:checkOnline(name)

-- checks if player is online and puts information into an online buffer.

]]--
function Friendlist:checkOnline(name)
	local isOnline = 0;

	--FlagRSP:printDebug("checking online state for: " .. name);

	for i=1, GetNumFriends() do
		local n, level, class, area, connected = GetFriendInfo(i);

		if n == name then
			if connected then
				isOnline = 1;
			else 
				isOnline = -1;
			end
		end

		--if name == FlagRSP.pName then
			--onlineInfo = "|cff00c400" .. FRIENDLIST_LOCALE_OnlineLine .. "|r";
		--end
	end

	if isOnline == 0 and flagRSP_IntervalListHigh[name] ~= nil and flagRSP_LastSeenListHigh[name] ~= nil then
		--FlagRSP:printDebug("check if " .. name .. " is online. Now it is: " .. math.ceil(GetTime()) .. " and user will post again at " .. math.ceil(flagRSP_IntervalListHigh[name]+flagRSP_LastSeenListHigh[name]));
		if GetTime() > flagRSP_IntervalListHigh[name]*2+flagRSP_LastSeenListHigh[name] then
			--isOnline = false; 
		else
			isOnline = 1; 
		end
	elseif isOnline == 0 and flagRSP_IntervalList[name] ~= nil and flagRSP_LastSeenList[name] ~= nil then
		if GetTime() > flagRSP_IntervalList[name]*2+flagRSP_LastSeenList[name] then
			--isOnline = false; 
		else
			isOnline = 1; 
		end
	end

	if ( name ~= nil ) then
		if self.onlineBufferList[name] == nil then
			self.onlineBufferList[name] = {};
		end

		self.onlineBufferList[name].isOnline = isOnline;
		self.onlineBufferList[name].timeStamp = GetTime();
	end

	return isOnline;
end


--[[

Friendlist:isOnline(name)

-- returns true if player <name> is online.

]]--
function Friendlist:isOnline(name)
	local isOnline = 0;

	if self.onlineBufferList[name] ~= nil and (GetTime()-self.onlineBufferList[name].timeStamp) <= 10  then
		isOnline = self.onlineBufferList[name].isOnline;
	else 
		isOnline = self.checkOnline(name);
	end

	return isOnline;
end


--[[

Friendlist:GetOnlineInfo(name)

-- Get information if player name is online.

]]--
function Friendlist:GetOnlineInfo(name)
	local onlineInfo = "";
	local isOnline = self:isOnline(name);

	if isOnline == 1 then
		onlineInfo = "|cff00c400" .. FRIENDLIST_LOCALE_OnlineLine .. "|r";
	elseif isOnline == -1 then
		onlineInfo = "|cff808080" .. FRIENDLIST_LOCALE_OfflineLine .. "|r";
	end

	return onlineInfo;
end


--[[

Friendlist:GetOnlineLocation(name)

-- Get information where player name currently is.

]]--
function Friendlist:GetOnlineLocation(name)
	local onlineLoc = "";

	for i=1, GetNumFriends() do
		local n, level, class, area, connected = GetFriendInfo(i);

		if n == name then
			if connected then
				onlineLoc = area;
			end
		end
	end

	if name == FlagRSP.pName then
		onlineLoc = GetRealZoneText();
	end

	return onlineLoc;
end


--[[

Friendlist:GetOnlineStatus(name)

-- Get status line for player's online status.

]]--
function Friendlist:GetOnlineStatus(name, playerID)
	local status = "";

	if self:GetType(name) == "char" then
		local onlineInfo = self:GetOnlineInfo(name);
		if onlineInfo ~= "" then
			local onlineLoc = self:GetOnlineLocation(name);
			if onlineLoc ~= "" then
				status = "[" .. onlineInfo .. ", " .. onlineLoc .. "]";
				--TooltipHandler:ParseString(FRIENDLIST_LOCALE_OnlineStatusLoc, playerID);
			else
				status = "[" .. onlineInfo .. "]";
				--TooltipHandler:ParseString(FRIENDLIST_LOCALE_OnlineStatusNoLoc, playerID);
			end
		end
	end

	return status;
end

--[[

Friendlist:setButtonPosition(pos)

- Set position of Minimap button.
-- TODO: Change this to be handled by the FlagRSPConfigure methods.

]]--
function Friendlist:setButtonPosition(pos)
	-- Save position.
	Friendlist_Settings[Friendlist.realmName][Friendlist.playerName].buttonPosition = pos;

	-- Convert degree to radiant.
	local pos_rad = pos * 2 * (3.1415926536 / 360);
	-- Raduis of Minimap
	--r = 82.735;
	local r = 81.500;
	--r = 93.735;
   
	-- Polar coordinates, yummy.
	local x = math.floor(r * math.cos(pos_rad)+0.5)-12;
	local y = math.floor(r * math.sin(pos_rad)+0.5)+12;

	FRIENDLISTMinimapButton:SetPoint("TOPLEFT", "Minimap", "CENTER", x, y);
end

--[[

Friendlist:updateMainFrame()

- Updates the settings (checkboxes) in the main frame of Friendlist.

]]--
function Friendlist:updateMainFrame()
end

-- Import all contacts from wow friendlist
-- TODO: Reimplement this.
function ImportFriendlist()
	Print_Chat("Import Friendlist...");
	for i=1, GetNumFriends(), 1 do
		local name, level, class, area, connected = GetFriendInfo(i);
		if(name) then 
			if(name == "Unbekannt") then
			else 
				if(friendData[Friendlist.realmName][Friendlist.playerName].FRIEND[name] == nil) then
					local friend = { };
					friend.name = name;
					Friendlist:AddFriend(friend);
					-- Don't set friendstate...
				end -- if(not friendData...
			end -- if(name!=...
		end -- if(name)...
	end -- for i...
   
	-- Update UI
	FRIENDLIST_Update();
end -- ImportFriendlist()

------------------------------------------------------------------------------------------

-- Export friends to wow contactlist
-- TODO: Reimplement this.
function ExportFriendlist() 
	Print_Chat("Export Friendlist...");
	if ( (friendData == nil) or
		(friendData[Friendlist.realmName] == nil) or
		(friendData[Friendlist.realmName][Friendlist.playerName] == nil) or
		(friendData[Friendlist.realmName][Friendlist.playerName].FRIEND == nil) or 
		(friendData[Friendlist.realmName][Friendlist.playerName].numFRIEND == 0) ) then
		Print_Chat("FRIEND Data is empty - nothing to export");
	else
		-- iterate through frendlist
		for key, value in pairs(friendData[Friendlist.realmName][Friendlist.playerName].FRIEND) do		
			local found = false;
			-- only add, if he doesn't exist allready
			for i=1, GetNumFriends(), 1 do 
				local name, level, class, area, connected = GetFriendInfo(i);
				if(name == key) then 
					found = true;
				end -- if(name...
			end -- for i=1
			if(found == false) then
				AddFriend(key);
			end -- if(found...
		end -- for key, value...
	end -- if((frienddata==nil)...
   
	-- Update UI
	FriendsList_Update();
end -- ExportFriendlist()


------------------------------------------------------------------------------------------

function addfriend_Msg(msg)
	if (msg == "") or (msg == nil) then
		Print_Chat("Friend List version: "..FRIENDLIST_VERSION_NUM);
		Print_Chat("type /list for more help");	
	else
		local idIndex = 1;
		local word = nil;
		local name = nil;
		local nachnameString = "";
		local noteString = "";
		for word in string.gmatch(msg, "%S+") do
			if (idIndex == 1) then
				name = word;
			elseif (idIndex == 2) then
				nachnameString = word;
			elseif (idIndex == 3) then
				noteString = word;
			elseif (idIndex > 3) then
				noteString = noteString .. " " .. word;
			end
			idIndex = idIndex + 1;
		end
      
		if ( (idIndex == 2) ) then
			nachnameString = nil;
			noteString = nil;
		end
		if (name ~= nil) then
			local friend = { };
			
			friend.name = name;
			friend.surname = nachnameString;
			friend.notes = noteString;
			
			self:AddFriend(friend);
		end
	end
end

-----------------------------------------------------------------------------------------
--[[

Below this comment block lies the accessor methods.  These methods are the only methods
allowed to directly interact with the friendData table.  Doing it this way allows me to
mark a line point that the friendData table should not appear above in order to keep the
code base clean.

]]--
-----------------------------------------------------------------------------------------

--[[

Friendlist:isFriend(name)

- Check if player/object/guild is on Friendlist.

]]--
function Friendlist:isFriend(name)
	--We can't check our Friendlist list if the variables aren't loaded or
	--our data structure isn't created yet
	if ( name == nil or name == "" ) then
		return false;
	end

	--If we get here, we are initialized and have valid data...check for FRIEND status	
	if( FriendlistData.Friends[name] == nil ) then
		return false;
	else
		return true;
	end
end

--[[

Friendlist:GetName(playerID, first)

-- Accepts an index from the Friendlist and returns the character's name
   and Given name, but if first is specified, returns just the first name.

]]--
function Friendlist:GetName(playerID, first)
	local index = tonumber(string.sub(playerID, 6, string.len(playerID)));
	local unitName, firstName;
	
	if index ~= nil then
		for key, value in pairs(FriendlistData.Friends) do
			if ( value.index ~= nil and index == value.index ) then
				-- we found our entry, key is wanted name
				unitName = key;
				
				if ( value.altName ~= nil and value.altName ~= "" ) then
					-- Character has an alternate first name.
					firstName =  value.altName
				end
				
				if ( first == nil ) then
					-- Return both names, unitName and firstName.
					return unitName, firstName;
				else
					-- Return just the first name.
					return firstName;
				end
			end
		end
	end
end

--[[

Friendlist:GetAltName(name)

-- Returns an alternate name for <name>

]]--
function Friendlist:GetAltName(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].altName;
	end
end

--[[

Friendlist:SetAltName(name, altName)

-- Update and alternate name for <name>

]]--
function Friendlist:SetAltName(name, altName)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].altName = altName;
	end
end

--[[

Friendlist:GetSurname(name)

- Get player names surname.

]]--
function Friendlist:GetSurname(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].surname;
	end
end

--[[

Friendlist:SetSurname(name, surname)

-- Update surname for entry <name>.

]]--
function Friendlist:SetSurname(name, surname)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].surname = surname;
	end
end

--[[

Friendlist:GetGuild(name)

-- Get player name's guild.

]]--
function Friendlist:GetGuild(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].guild;
	end
end

--[[

Friendlist:SetGuild(name, guild)

-- Update guild for entry <name>.

]]--
function Friendlist:SetGuild(name, guild)	
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].guild = guild;
	end
end

--[[

Friendlist:GetLevel(name)

-- Get player name's level.

]]--
function Friendlist:GetLevel(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		-- Although guilds will not be supported in normal view for future releases we have to capture their level for now.
		if ( self:GetType(name) == "char" ) then
			return FriendlistData.Friends[name].level;
		else
			return "";
		end
	else 
		return "";
	end
end

--[[

Friendlist:SetLevel(name, level)

-- Update guild for entry <name>.

]]--
function Friendlist:SetLevel(name, level)
	if FriendlistData.Friends[name] ~= nil then
		FriendlistData.Friends[name].level = level;
	end
end

--[[

Friendlist:GetClass(name)

-- Get player name's class.

]]--
function Friendlist:GetClass(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].class;
	end
end

--[[

Friendlist:SetClass(name, class)

-- Update guild for entry <name>.

]]--
function Friendlist:SetClass(name, class)
	if FriendlistData.Friends[name] ~= nil then
		FriendlistData.Friends[name].class = class;
	end
end

--[[

Friendlist:GetRace(name)

-- Get player name's race.

]]--
function Friendlist:GetRace(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].race;
	end
end

--[[

Friendlist:SetRace(name, race)

-- Update race for entry <name>.

]]--
function Friendlist:SetRace(name, race)
	if FriendlistData.Friends[name] ~= nil then
		FriendlistData.Friends[name].race = race;
	end
end

--[[

Friendlist:GetRank(name)

-- Get player name's guild.

]]--
function Friendlist:GetRank(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].rank;
	end
end

--[[

Friendlist:SetRank(name, rank)

-- Update race for entry <name>.

]]--
function Friendlist:SetRank(name, rank)
	--FlagRSP:printDebug("Rank: " .. rank);
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].rank = rank;
	end
end

--[[

Friendlist:GetTitle(name)

-- Get player name's title.

]]--
function Friendlist:GetTitle(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].title;
	end
end

--[[

Friendlist:SetTitle(name, title)

-- Update surname for entry <name>.

]]--
function Friendlist:SetTitle(name, title)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].title = title;
	end
end

--[[

Friendlist:SetFriendstate(name, friendstate)

-- Set the friendstate at entry name.

]]--
function Friendlist:SetFriendstate(name, friendstate)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].friendstate = friendstate;
	end
end

--[[

Friendlist:GetType(name)

- Get type for entry <name>.

]]--
function Friendlist:GetType(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].type;
	end
end

--[[

Friendlist:SetType(name, type)

-- Sets type to <type> at entry <name>.

]]--

function Friendlist:SetType(name, type)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].type = type;
	end
end

--[[

Friendlist:GetNotes(name)

- Get player names notes.

]]--

function Friendlist:GetNotes(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].notes;
	end
end

--[[

Friendlist:SetNotes(name, notes)

-- Sets the player notes to <notes> at <name>

]]--

function Friendlist:SetNotes(name, notes)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].notes = notes;
	end
end

--[[

Friendlist:GetFriendstate(name)

- Get player names friend status (currently only -10 for foe and 10 for friend).

]]--
function Friendlist:GetFriendstate(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].friendstate;
	end
end

--[[

Friendlist:SetFriendstate(name, friendstate)

-- Sets the friendstate for <name> to <friendstate>.

]]--

function Friendlist:SetFriendstate(name, friendstate)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].friendstate = friendstate;
	end
end

--[[

Friendlist:GetIndex(name)

- Returns index for entry <name>.

]]--
function Friendlist:GetIndex(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].index;
	end
end

--[[

Friendlist:SetIndex(name, index)

-- Sets the index at <name> to <index>.

]]--
function Friendlist:SetIndex(name, index)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].index = index;
	end
end

--[[

Friendlist:GetFriends()

-- Returns the number of friends the player has in their Friendlist.
-- This AddOn, not the default Friends Lists.

]]--
function Friendlist:GetFriends()
	return FriendlistData.numFriends;
end

--[[

Friendlist:SetFriends(number)

-- Sets the number of friends in the Friendlist to <number>.

]]--
function Friendlist:SetFriends(number)
	FriendlistData.numFriends = number;
end

--[[

Friendlist:AddFriend(friend)

- Due to the fact that adding new data to the FriendlistData table requires editing of countless
  functions in order to add support for one new field, this function accepts a table containing
  relevant friend data and then stores it in the friendData table.

]]--
function Friendlist:AddFriend(friend)
	-- We need a name to store this information, if it's not there, exit.
	if ( friend.name == nil or friend.name == "" ) then
		return
	end
	
	-- Initialize all the data to empty strings if it's been passed as nil.
	if ( friend.notes == nil ) then
		friend.notes = "";
	end
	if ( friend.surname == nil ) then
		friend.surname = "";
	end
	if ( friend.title == nil ) then
		friend.title = "";
	end
	if ( friend.friendstate == nil ) then
		friend.friendstate = 0; -- Defaults to known.
	end
	if ( friend.newName == nil ) then
		friend.newName = ""; -- If this value were set, this would be the previous entry to edit.
	end
	if ( friend.altName == nil ) then
		friend.altName = "";
	end
	if ( friend.type == nil or friend.type == "" ) then
		friend.type = "char"; -- Friend is a character, not a guild.
	end
	
	-- Now, we add the data into the table.  
	if ( friend.newName == "" ) then
		-- We're adding a new entry, intialize it first.
		self:InitializeFriend(friend.name);
		
		-- Add in the basic information.
		self:SetSurname(friend.name, friend.surname);
		self:SetAltName(friend.name, friend.altName);
		self:SetNotes(friend.name, friend.notes);
		self:SetTitle(friend.name, friend.title);
		self:SetFriendstate(friend.name, friend.friendstate);
		self:SetType(friend.name, friend.type);
		
		-- Initialize these to empty values and let the normal update code handle them.
		self:SetClass(friend.name, "");
		self:SetLevel(friend.name, 0);
		self:SetRace(friend.name, "");
		self:SetGuild(friend.name, "");
		
		-- Print out successful adding of friend.
		FlagRSP:printM(string.gsub(FRIENDLIST_LOCALE_AddFriendMsg, "%%n", friend.name));
	else
		-- Editing an old entry.  Shouldn't I have an edit function do this?
		-- Hey, what do you know, I do.  Keep this until I've converted all edits to use the edit function.
		self:SetSurname(friend.name, friend.surname);
		self:SetAltName(friend.name, friend.altName);
		self:SetNotes(friend.name, friend.notes);
		self:SetTitle(friend.name, friend.title);
		self:SetFriendstate(friend.name, friend.title);
		
		if ( friend.newName ~= "" and friend.newName ~= nil and friend.newName ~= friend.name ) then
			self:MoveFriend(friend.newName, friend.name);
		end
	end
	
	-- Calling this for safety for now, when complete, look through and see if this can be fixed up.
	FRIENDLIST_Update();
end

--[[

Friendlist:InitializeData()

-- Creates a new FriendlistData structure.
-- Warning: This doesn't care if FriendData already exists.

]]--
function Friendlist:InitializeData()
	FriendlistData = { };
	FriendlistData.Friends = { };
	
	self:SetFriends(0);
end

--[[

Friendlist:InitializeFriend(name)

-- Initializes entry in friendData[realmName][playerName].FRIEND[name].
-- If the entry already exists, return immediately, otherwise initialize the entry.

]]--
function Friendlist:InitializeFriend(name)
	if ( name == nil or name == "" ) then
		return;
	end
	
	-- Test to make sure the friend doesn't exist.
	if ( FriendlistData.Friends[name] == nil ) then
		FriendlistData.Friends[name] = { };
		
		-- This data is necessary for the FauxScrollFrame to render the Friendlist correctly.
		local newIndex = self:GetFriends() + 1;
		self:SetIndex(name, newIndex);
		self:SetFriends(newIndex);
	end
end

--[[

Friendlist:MoveFriend(new, old)

-- Copies the friend entry at old to the friend entry at new and deletes the entry at old.

]]--
function Friendlist:MoveFriend(new, old)
	if ( FriendlistData.Friends[old] == nil ) then
		return;
	end
	
	-- Check to see that someone with this name doesn't exist.
	if ( FriendlistData.Friends[new] ~= nil ) then
		-- If the indices are different, then they are two different individuals, old friend can't move in.
		if ( self:GetIndex(new) ~= self:GetIndex(old) ) then
			FlagRSP:printE(string.gsub(FRIENDLIST_LOCALE_NameExistsMsg,"%%n",new));
			return;
		end
	end
	
	local oldFriend = FriendlistData.Friends[old];
	FriendlistData.Friends[new] = oldFriend;
	FriendlistData.Friends[old] = nil;
	
	FlagRSP:printM(string.gsub(FRIENDLIST_LOCALE_MoveFriendMsg, "%%n", new));
end

--[[

Friendlist:EditFriend(friend)

-- Parameters:
--   friend - Table containing friend data.
-- Edits the friend data at name to contain the values in friend.
-- If friend.newName is set, MoveFriend will be called.

]]--
function Friendlist:EditFriend(friend)
	-- The previous name or the character's current name needs to exist.
	if ( friend.name == nil or friend.name == "" ) then
		return;
	end
	
	-- Initialize all the data to empty strings if it's been passed as nil.
	if ( friend.notes == nil ) then
		friend.notes = "";
	end
	if ( friend.surname == nil ) then
		friend.surname = "";
	end
	if ( friend.title == nil ) then
		friend.title = "";
	end
	if ( friend.friendstate == nil ) then
		friend.friendstate = 0; -- Defaults to known.
	end
	if ( friend.altName == nil ) then
		friend.altName = "";
	end
	
	self:SetSurname(friend.name, friend.surname);
	self:SetAltName(friend.name, friend.altName);
	self:SetNotes(friend.name, friend.notes);
	self:SetTitle(friend.name, friend.title);
	self:SetFriendstate(friend.name, friend.friendstate);
	
	-- Test to see if a new name has been passed.
	if ( friend.newName ~= nil and friend.newName ~= "" and friend.newName ~= friend.name ) then
		-- Move the friend up, to the east side.
		self:MoveFriend(friend.newName, friend.name);
	end
end

--[[

Friendlist:DeleteFriend(name)

-- Deletes entry at <name> and decrements the numFRIEND and reorders the indices.

]]--
function Friendlist:DeleteFriend(name)
	if ( name == nil or name == "" ) then
		return;
	end
	
	-- Perserver the old index.
	local oldIndex = self:GetIndex(name);
	
	-- Break up with the old friend.
	FriendlistData.Friends[name] = nil;
	self:SetFriends(self:GetFriends() - 1);
	FlagRSP:printM(string.gsub(FRIENDLIST_LOCALE_DelFriendMsg, "%%n", name));
	
	for key in pairs(FriendlistData.Friends) do
		if ( self:GetIndex(key) > oldIndex ) then
			self:SetIndex(key, self:GetIndex(key) - 1);
		end
	end
end

--[[

Friendlist:GetFriend(index)

-- Returns a table containing friend info retrieved from the Friendlist:Get*() functions.
-- I'd rather do it that way instead of just returning the table at Friends[name] in case
-- I change the keys to something stupid in a fit of madness.

]]--
function Friendlist:GetFriend(index)
	local friend = { };
	
	for key, value in pairs(FriendlistData.Friends) do
		if ( self:GetIndex(key) == index ) then
			friend.name = key;
			friend.surname = self:GetSurname(key);
			friend.altName = self:GetAltName(key)
			friend.class = self:GetClass(key);
			friend.guild = self:GetGuild(key);
			friend.level = self:GetLevel(key);
			friend.friendstate = self:GetFriendstate(key);
			friend.notes = self:GetNotes(key);
		end
	end
	
	return friend;
end

--[[

Friendlist:GetFriendsTable()

-- This returns the table that stores all the friend information.
-- This is used for the sorting functions in the Friendlist.

]]--
function Friendlist:GetFriendsTable()
	if ( FriendlistData.Friends ~= nil ) then
		return FriendlistData.Friends;
	end
end

--[[

Friendlist:ConvertToPerCharacterFriendlist()

-- Holy long method name, Batman!  It converts the old global data structure
   into one saved on a per character basis.  This would, in theory, clean up a lot
   of code and make it so I don't have to constantly test if playerName and realmName
   are initialized before checking the data structure, since they won't be needed.
   We don't nil this out like in the other conversions because this data sits in
   an extra mod that can be disabled.

]]--

function Friendlist:ConvertToPerCharacterFriendlist()
	FriendlistData = { };

	if ( friendData[FlagRSP.rName][FlagRSP.pName].FRIEND == nil ) then
		return;
	end
	
	FriendlistData.Friends = { };
	
	-- Loop through the old friendData array and copy the friends.
	for key, value in pairs(friendData[FlagRSP.rName][FlagRSP.pName].FRIEND) do
		if ( FriendlistData.Friends[key] == nil ) then
			FriendlistData.Friends[key] = value;
		end
	end
	
	FriendlistData.numFriends = friendData[FlagRSP.rName][FlagRSP.pName].numFRIEND;
	friendData[FlagRSP.rName][FlagRSP.pName].converted = true;
end

--[[

Friendlist:ShowUnconverted()

-- This loops through the old friendData structure and reports which characters have not been converted.

]]--

function Friendlist:ShowUnconverted()
	for key in pairs(friendData) do
		for key2 in pairs(friendData[key]) do
			if ( friendData[key][key2] ~= nil ) then
				if ( friendData[key][key2].converted ~= true and friendData[key][key2].FRIEND ~= { } ) then
					Print_Chat(key2 .. " on " .. key .. " needs to be converted.");
				end
			end
		end
	end
end

--[[

Friendlist:ResetFriendlist()

-- Empties the Friends element and set the amount of Friends to 0.

]]--

function Friendlist:ResetFriendlist()
	FriendlistData.Friends = { };
	FriendlistData.SetFriends(0);
end

--[[

Friendlist:ResetFriends()

-- Clears out the stored data stored for the the friends, but keeps their entries.

]]--

function Friendlist:ResetFriends()
	for key in pairs(FriendlistData.Friends) do
		self:SetNotes(key, "");
		self:SetClass(key, "");
		self:SetSurname(key, "");
		self:SetGuild(key, "");
		self:SetLevel(key, 0);
		self:SetFriendstate(key, 0);
	end
end

--[[

Friendlist:PrintFriendlist()

-- Prints out the contents of the Friendlist to the chat frame.

]]--

function Friendlist:PrintFriendlist()
	for key, value in pairs(FriendlistData.Friends) do
		Print_Chat("Name: " .. key);
		Print_Chat("   Surname: " .. self:GetSurname(key));
		Print_Chat("   Given Name: " .. self:GetAltName(key));
		Print_Chat("   Class: " .. self:GetClass(key));
		Print_Chat("   Guild: " .. self:GetGuild(key));
		Print_Chat("   Notes: " .. self:GetNotes(key));
		Print_Chat("   Level: " .. self:GetLevel(key));
	end
end
