-- flagRSP
--
-- based on xTooltip by Author : Per 'Doxxan' Lnn Wege
--

--[[

TODO: Migrate all functions to the FlagRSP object.

]]--

xTP_ChannelName = "xtensionxtooltip2";

flagRSP_CharStatusList = {};
flagRSP_IntervalList = {};
flagRSP_LastSeenList = {};
flagRSP_IntervalListHigh = {};
flagRSP_LastSeenListHigh = {};

--~ FlagRSP_CharDesc = {};

--~ FlagRSP_CharDesc.availRev = {};
--~ FlagRSP_CharDesc.savedRev = {};
--~ FlagRSP_CharDesc.savedDesc = {};

flagRSP_Debug = false;
flagRSP_AFK = false;
flagRSP_AFKPosx = 0;
flagRSP_AFKPosy = 0;

--[[

flagRSP constants

]]--
FLAGRSP_MININTERVAL = 120;
FLAGRSP_MAXINTERVAL = 7200;
FLAGRSP_MININTERVALHIGH = 30;
FLAGRSP_MAXINTERVALHIGH = 1000;
FLAGRSP_TARGETMPM = 60;

--[[

flagRSP library functions.

]]--
FlagRSP = {
	oldTick = 0;
	numMessagesInInterval = 0;

	bytesInInterval = 0;
	bytesPerSecondLastInterval = 0;
	bytesPerSecondIntervalMax = 0;
	bytesPerSession = 0;

	ownDescriptionSent = 0;
	ownDescriptionRequested = 0;
	maxUsersOnline = 0;

	oldestVersion = "";
	newestVersion = "";

	sessionStartTick = 0;
	isInitialized = false;

	maxPostingCost = 147;
	maxPostingCostHigh = 42;
	postingTargetBps = 25;
	postingTargetBpsHigh = 25;

	oldTickChatFrame = 0;

	postInterval = 2000;
	postIntervalHigh = 400;
	displayBoxInterval = 10;
	rescaleBoxInterval = 120000;
	rescaleBoxBtnInterval = 120000;
	
	startTime = 0;
	
	postFlagInterval = 60;
	postFlagLast = 0;
	
	canPost = false;

	newTickChatFrame = 0;
	newTickPost = 0;
	newTickPostHigh = 0;
	newTickDesc = 0;
	newTickDisplayBox = 0;
	newTickRescaleBox = {
		FlagRSPInfoBox = 0,
		FRIENDLISTFrameInfoBox = 0
	},
	newTickRescaleBoxBtn = 0;

	updateFriendsTick = 0;
	updateGuildListTick = 0;
	updateIgnoredListTick = 0;

	pName = "";
	rName = "";

	releaseVersion = 0;
	majorVersion = 6;
	minorVersion = 0;
	versionString = "0.6.0";

	isStableVersion = true;

	VERSION = "";

	notificationSeen = false;
   
	playerEnteredWorld = false;

	standby = false;
	collecting = false;

	displayOwnTooltip = false;
};

flagRSPDetails = {
	name = "flagRSP",
	version = FlagRSP.versionString,
	author = "",
	email = "",
	website = "",
	category = MYADDONS_CATEGORY_OTHERS
};

flagRSPHelp = {};
flagRSPHelp[1] = "For help regarding flagRSP please look into the documentation files contained in <WoW>\\Interface\\AddOns\\flagRSP\\documentation.";


--[[

FlagRSP:getHexString(num)

- Returns hex number as string with minimum string length len for positive integer num.

]]--
function FlagRSP:getHexString(num, len)
	local digits = "0123456789ABCDEF";
	local hexnum = "";
	local intdigit;

	while num > 0 do
		num = math.floor(num);
		intdigit = math.fmod(num, 16);
		num = math.floor(num/16);

		hexnum = string.sub(digits, (intdigit+1), (intdigit+1)) .. hexnum;
	end

	while string.len(hexnum) < len do
		hexnum = "0" .. hexnum;
	end
   
	return hexnum;
end


function FlagRSP:simplehash(s)
	local a = 378551;
	local b = 63689;
	local h = 0;

	for i=1, string.len(s) do
		h = h * a + string.byte(s,i);
		a = a * b;
	end

	return h;
end

--[[

FlagRSP:GetHex(red, green, blue)

-- Takes in three decimal color values and outputs one hex string.
   Mostly just to save typing space, however this function will be used
   more often.

]]--
function FlagRSP:GetHex(red, green, blue)
	local hexstring = "";
	
	hexstring = self:getHexString(red*255, 2) .. self:getHexString(green*255, 2) .. self:getHexString(blue*255, 2);
	
	return hexstring;
end


--[[

FlagRSP:getVersionNum(ver)

- Converts version string <ver> into numbers.

]]--
function FlagRSP:getVersionNum(ver)
	local release, major, minor, dotPos;

	dotPos = string.find(ver, "%.");
	if dotPos ~= nil then 
		release = tonumber(string.sub(ver, 1, dotPos-1)); 
		ver = string.sub(ver, dotPos+1, string.len(ver));
	else 
		release = -1; 
	end
   
	dotPos = string.find(ver, "%.");
	if dotPos ~= nil then 
		major = tonumber(string.sub(ver, 1, dotPos-1));
		ver = string.sub(ver, dotPos+1, string.len(ver));
	else
		major = -1; 
	end

	minor = tonumber(string.sub(ver, 1, string.len(ver)));

	if minor == nil then
		minor = -1;
	end
   
   
	return release, major, minor;
end



--[[

FlagRSP:checkVersion(ver)

- Checks for a new version of flagRSP.

]]--
function FlagRSP:checkVersion(ver)
	--oldestVersion = "";
	--newestVersion = "";

	local release, major, minor = self:getVersionNum(ver);
   
	local oR, oMj, oMn = self:getVersionNum(self.oldestVersion);
	local nR, nMj, nMn = self:getVersionNum(self.newestVersion);

	--self:printDebug("version string is: " .. ver);
  
	local newVersion = false;

	if release ~= nil and major ~= nil and minor ~= nil and release >= 0 and major >= 0 and minor >= 0 then
		if release > self.releaseVersion then
			newVersion = true;
		elseif major > self.majorVersion and release == self.releaseVersion then
			newVersion = true;
		elseif minor > self.minorVersion and major == self.majorVersion and release == self.releaseVersion then
			newVersion = true;
		end

		if release > nR then
			self.newestVersion = ver;
		elseif major > nMj and release == nR then
			self.newestVersion = ver;
		elseif minor > nMn and major == nMj and release == nR then
			self.newestVersion = ver;
		end

		if oR == -1 or oMj == -1 or oMn == -1 then
			self.oldestVersion = ver;
		end
      
		if release < oR then
			self.oldestVersion = ver;
		elseif major < oMj and release == oR then
			self.oldestVersion = ver;
		elseif minor < oMn and major == oMj and release == oR then
			self.oldestVersion = ver;
		end      
	end


	if newVersion then      
		self:printDebug("New version available");
		self:showVersionNotification(release, major, minor);
	end
end

function FlagRSP:getRPFlag()
	if ( FlagRSPConfigure:GetRPFlag() == nil ) then
		self:setRPFlag(0);
	end

	return FlagRSPConfigure:GetRPFlag();
end


function FlagRSP:setCharStatus(status)
	local mesg;
	local changed = false;

	if status ~= FlagRSPConfigure:GetCharacterStatus() then
		if status == "ooc" then
			mesg = FlagRSP_Locale_OOCFlag;
		elseif status == "ic" then
			mesg = FlagRSP_Locale_ICFlag;
		elseif status == "ffa-ic" then
			mesg = FlagRSP_Locale_FFAICFlag;
		elseif status == "none" then
			mesg = FlagRSP_Locale_NoCFlag;
		elseif status == "st" then
			mesg = FlagRSP_Locale_STFlag;
		end
		
		FlagRSPConfigure:SetCharacterStatus(status);
      
		self:printM(mesg);
		changed = true;
	end

	return changed;
end 


function FlagRSP:getCharStatus()
	if FlagRSPConfigure:GetCharacterStatus() == nil then
		self:setCharStatus("none");
	end

	return FlagRSPConfigure:GetCharacterStatus();
end

function FlagRSP:getCharStatusNum()
	if FlagRSPConfigure:GetCharacterStatus() == nil then
		self:setCharStatus("none");
	end

	local num = 0; 
	local cStatus = FlagRSPConfigure:GetCharacterStatus();

	if cStatus == "none" then
		num = 0;
	elseif cStatus == "ooc" then
		num = 1;
	elseif cStatus == "ic" then
		num = 2;
	elseif cStatus == "ffa-ic" then
		num = 3;
	elseif cStatus == "st" then
		num = 4;
	end

	return num;
end

function FlagRSP:getOwnSurname()
	if ( FlagRSPConfigure:GetSurname() == nil ) then
		self:setSurname("");
	end

	return FlagRSPConfigure:GetSurname();
end

function FlagRSP:GetPlayerSurname()

end

function FlagRSP:GetPlayerName()
	if ( FlagRSPConfigure:GetName() == nil ) then
		self:SetName("");
	end
	
	return FlagRSPConfigure:GetName();
end

function FlagRSP:GetPlayerMiddleName()
--	if ( FlagRSPData.MiddleName == nil ) then
		self:SetMiddleName("");
--	end
	
--	return FlagRSPData.MiddleName;
	return "";
end

function FlagRSP:getOwnTitle()
	if FlagRSPConfigure:GetTitle() == nil then
		self:setTitle("");
	end

	return FlagRSPConfigure:GetTitle();
end

function FlagRSP:getOwnDesc()
	return FlagRSPConfigure:GetDescription();
end


function FlagRSP:setDesc(desc)
	local changed = false;

	if desc ~= FlagRSPConfigure:GetDescription() then
		FlagRSPConfigure:SetRevision(FlagRSPConfigure:GetRevision() + 1);
		FlagRSPConfigure:SetDescription(desc);

		self:printM(FlagRSP_Locale_DescUpdate);

		changed = true;
	end

	-- stop posting after change immediately.
	-- TODO: Stop posting only if current description has been changed.
	if changed then
		FlagRSPInfo.descPos = -1;
		-- TODO: Immediately update the local FlagList with the new description.
		--FlagRSP:PostSelfDescription();
	end

	return changed;
end

function FlagRSP:SetName(name)
	local changed = false;
	
	if ( string.len(name) > 50 ) then
		name = string.sub(name, 1, 50);
	end
	
	if ( name ~= FlagRSPConfigure:GetName() ) then
		FlagRSPConfigure:SetName(name);
		
		if ( name == " " or name == "" or name == UnitName("player") ) then
			-- Print a message or something saying no first name.
			local msg = string.gsub(FlagRSP_Locale_NoAltName, "%%s", UnitName("player"));
			self:printM(msg);
		else
			-- Print another message that you have an alternate first name.
			local msg = string.gsub(FlagRSP_Locale_NewAltName, "%%s", name);
			self:printM(msg);
		end
		
		changed = true;
	end
	
	return changed;
end

function FlagRSP:SetMiddleName(name)
	local changed = false;
	
	if ( string.len(name) > 50 ) then
		name = string.sub(name, 1, 50);
	end
	
	if ( name ~= FlagRSPData.MiddleName ) then
		FlagRSPData.MiddleName = name;
		
		if ( name == " " or name == "" ) then
			-- Print a message or something saying no middle name.
			
		else
			-- Print another message that you have no middle name.
			
		end
		
		changed = true;
	end
	
	return changed;
end

function FlagRSP:setSurname(name)
	local changed = false;

	if string.len(name) > 50 then
		name = string.sub(name,1,50);
	end

	if name ~= FlagRSPConfigure:GetSurname() then
		FlagRSPConfigure:SetSurname(name);
      
		if name == " " or name == "" then
			self:printM(FlagRSP_Locale_NoSurname);
		else
			local msg = string.gsub(FlagRSP_Locale_NewSurname, "%%s", name);
			self:printM(msg);
		end

		changed = true;
	end

	return changed;
end

function FlagRSP:setTitle(title)
	local changed = false;

	if string.len(title) > 50 then
		title = string.sub(title,1,50);
	end
	if title ~= FlagRSPConfigure:GetTitle() then
		FlagRSPConfigure:SetTitle(title);

		if title == " " or title == "" then
			self:printM(FlagRSP_Locale_NoTitle);
		else
			local msg = string.gsub(FlagRSP_Locale_NewTitle, "%%s", title);
			self:printM(msg);
		end
      
		changed = true;
	end

	return changed;
end   


   
function FlagRSP:enableRPTags()
	self:setRPFlag(5);
	self:setNameDisp(true);
	FlagRSPConfigure:SetLevelDisplay("relative");
   
	-- flokru: Name and title notification, a bit quick and dirty.
	if self:getOwnSurname() ~= nil then
		self:setSurname(self:getOwnSurname());
	end
	if self:getOwnTitle() ~= nil then
		self:setTitle(self:getOwnTitle());
	end
   
	self:PostFlags();
   
	--   if xTP_CustomTag[UnitName("player")] ~= NIL then
	--      if xTP_CustomTag[UnitName("player")] == " " or xTP_CustomTag[UnitName("player")] == "" then
	--	 DEFAULT_CHAT_FRAME:AddMessage(UnitName("player").. " besitzt keinen Nachnamen.");
	--      else
	--	 DEFAULT_CHAT_FRAME:AddMessage("Ihr seid bekannt als " ..UnitName("player").. " " ..xTP_CustomTag[UnitName("player")]);
	--     end
	--   end
   
	--   if xTP_CustomTag2[UnitName("player")] ~= NIL then
	--      if xTP_CustomTag2[UnitName("player")] == " " or xTP_CustomTag2[UnitName("player")] == "" then
	--	 DEFAULT_CHAT_FRAME:AddMessage("Ihr f\195\188hrt noch keinen Titel");
	--      else
	---	 DEFAULT_CHAT_FRAME:AddMessage("Ihr f\195\188hrt den Titel " ..xTP_CustomTag2[UnitName("player")]);
	--      end
	--   end
end
   
function FlagRSP:disableRPTags()
	self:setRPFlag(0);
	self:setNameDisp(false);
	self:setLevelDisp("none");

	-- flokru: However, we should give a notification here as well, so that
	--         players notice that there are still tags set.
	-- flokru: Name and title notification, a bit quick and dirty.
	if self:getOwnSurname() ~= nil then
		self:setSurname(self:getOwnSurname());
	end
	if self:getOwnTitle() ~= nil then
		self:setTitle(self:getOwnTitle());
	end
   
	self:PostFlags();
end


--[[

FlagRSP:toggleAFK(msg)

- Toggles AFK for player and stops channel posting.

]]--
function FlagRSP:toggleAFK(msg, sentAfkMessage)
   --[[
	if msg ~= nil then
		self:printDebug("AFK-Message: " .. msg);
	else
		self:printDebug("AFK-Message: ");
		msg = "";
	end
	if flagRSP_AFK then
		flagRSP_AFK = false;
		self:print(FlagRSP_Locale_AFK_Deactivated);
	else
		flagRSP_AFK = true;
		self:print(FlagRSP_Locale_AFK_Activated);

		flagRSP_AFKPosx, flagRSP_AFKPosy = GetPlayerMapPosition("player");
		self:printDebug("flagRSP: DEBUG: AFK: x_position: " .. flagRSP_AFKPosx .. " y_position: " .. flagRSP_AFKPosy);
	end
]]--

	--if sentAfkMessage then
		--SendChatMessage(msg, "AFK");
	--end
end


--[[

FlagRSP:checkAFK(chatAuthor)

- Checks if player is still afk. If player is moving or entering chat 
  messages afk will be disabled.  
- Argument chatAuthor will be checked against player name.

]]--
function FlagRSP:checkAFK(chatAuthor)
	local posx, posy = GetPlayerMapPosition("player");

	if flagRSP_AFK then
		--self:printDebug("Chat-Author: " .. chatAuthor);
		if chatAuthor ~= nil and chatAuthor ~= "" then
			self:printDebug("flagRSP: DEBUG: Chat author: " .. chatAuthor);
		end
      
		if chatAuthor == UnitName("player") then
			self:printDebug("flagRSP: DEBUG: Player sent chat message, AFK will be disabled.");
			self:toggleAFK("", false);
		end
      
		if posx ~= flagRSP_AFKPosx or posy~= flagRSP_AFKPosy then
			self:printDebug("flagRSP: DEBUG: Player moved, AFK will be disabled.");
			self:toggleAFK("", false);
		end
	end
end


--[[

FlagRSP:caughtChatMessage(arg1,arg2)

- Makes statistics on how much chat messages are in xtensionxtooltip2.

]]--
function FlagRSP:caughtChatMessage()
	self.numMessagesInInterval = self.numMessagesInInterval + 1;
	self.bytesInInterval = self.bytesInInterval + string.len(arg1) + string.len(arg2) + string.len(xTP_ChannelName);
	self.bytesPerSession = self.bytesPerSession + string.len(arg1) + string.len(arg2) + string.len(xTP_ChannelName);

	if self.oldTick == 0 then self.oldTick = GetTime(); end

	if GetTime() - self.oldTick > 360 then
		self.bytesPerSecondLastInterval = self.bytesInInterval/(GetTime() - self.oldTick);

		if self.bytesPerSecondIntervalMax < self.bytesPerSecondLastInterval then
			self.bytesPerSecondIntervalMax = self.bytesPerSecondLastInterval;
		end

		--self.bytesPerSession = self.bytesPerSession + self.bytesInInterval;
		self.bytesInInterval = 0;

		local t,o,oMax = self:getStats();

		--self:printDebug("online are: " .. o);

		local altinterval = self.maxPostingCost * o / self.postingTargetBps;
		local altintervalHigh = self.maxPostingCostHigh * o / self.postingTargetBpsHigh;

		--self:printDebug("alt. interval would be: " .. altinterval);

		local messagesPerMinute = self.numMessagesInInterval/(GetTime() - self.oldTick)*60;
		local messageQuota = messagesPerMinute/FLAGRSP_TARGETMPM;

		--self.postInterval = messageQuota * self.postInterval;
		self.postInterval = altinterval;
		self.postIntervalHigh = altintervalHigh;
      
		if self.postInterval<FLAGRSP_MININTERVAL then self.postInterval = FLAGRSP_MININTERVAL; end
		if self.postInterval>FLAGRSP_MAXINTERVAL then self.postInterval = FLAGRSP_MAXINTERVAL; end

		if self.postIntervalHigh<FLAGRSP_MININTERVALHIGH then self.postIntervalHigh = FLAGRSP_MININTERVALHIGH; end
		if self.postIntervalHigh>FLAGRSP_MAXINTERVALHIGH then self.postIntervalHigh = FLAGRSP_MAXINTERVALHIGH; end

		-- Debug message for the developer.
		self:printDebug("Chat channel usage statistics. Messages: " .. self.numMessagesInInterval .. "  Number of messages per minute (last 6 minutes): " .. messagesPerMinute .. " Quota is: " .. messageQuota .. " new postinterval: " .. self.postInterval);
		self:printDebug("Bytes per second in the last interval: " .. self.bytesPerSecondLastInterval);
		self:printDebug("Bytes per second (whole session): " .. self.bytesPerSession/(GetTime() - self.sessionStartTick));

		self.numMessagesInInterval = 0;
		self.oldTick = GetTime();
	end
end


--[[

FlagRSP:playerIsIgnored(name)

- Checks, if player name is ignored.
- Returns true if ignored, false if not.

]]--
function FlagRSP:playerIsIgnored(name)
	local isIgnored = false;

	for t = 1, GetNumIgnores() do
		local iName = GetIgnoreName(t);
		if iName == name then 
			isIgnored = true; 	
			break; 
		end
	end

	return isIgnored;
end


--[[

FlagRSP:muteDataChannel()

- Mutes the communication channel for flagRSP.

]]--
function FlagRSP:muteDataChannel()
	if not flagRSP_Debug then
		ChatFrame_RemoveChannel(ChatFrame1, xTP_ChannelName);
	--else
		--DEFAULT_CHAT_FRAME:AddMessage("debug?",1,1,0);
	end
end


--[[

FlagRSP:toggleRankDisp()

- Toggles displaying of PvP ranks.

]]--
function FlagRSP:toggleRankDisp()
	if not self:getRankDisp() then
		self:setRankDisp(true);
	else
		self:setRankDisp(false);
	end
end


--[[

FlagRSP:toggleGuildDisp()

- Toggles displaying of guild names in tooltip.

]]--
function FlagRSP:toggleGuildDisp()
	if self:getGuildDisp() == 0 then
		self:setGuildDisp(1);
	elseif self:getGuildDisp() == 1 then
		self:setGuildDisp(-1);
	elseif self:getGuildDisp() == -1 then
		self:setGuildDisp(0);
	end
end


--[[

FlagRSP:setRankDisp()

- Sets rank display.

]]--
function FlagRSP:setRankDisp(value)
	local mesg;

	if value == 0 then value = false; end
	if value == 1 then value = true; end
   
	if value ~= true and value ~= false then return; end

	if value then
		mesg = FlagRSP_Locale_ShowRanks;
		FlagRSPConfigure:SetPVPRanks(true);
	else
		mesg = FlagRSP_Locale_HideRanks;
		FlagRSPConfigure:SetPVPRanks(false);
	end

	self:printM(mesg);
end


--[[

FlagRSP:getRankDisp()

- Gets rank display option.

]]--
function FlagRSP:getRankDisp()
	return FlagRSPConfigure:GetPVPRanks();
end


--[[

FlagRSP:setGuildDisp()

- Sets guild display.

]]--
function FlagRSP:setGuildDisp(value)
	local mesg;

	if value == false then value = -1; end
	if value == true then value = 1; end

	if value ~= 1 and value ~= -1 and value ~= 0 then return; end

	if value == 1 then
		mesg = FlagRSP_Locale_ShowAllGuild;
		FlagRSPConfigure:SetGuildDisplay("all");
	elseif value == -1 then
		mesg = FlagRSP_Locale_HideGuild;
		FlagRSPConfigure:SetGuildDisplay("none");
	elseif value == 0 then
		mesg = FlagRSP_Locale_ShowKnownGuild;
		FlagRSPConfigure:SetGuildDisplay("known");
	end

	self:printM(mesg);
end


--[[

FlagRSP:getGuildDisp()

- Gets guild display option.
-- Dirty hack to keep this compatible.
-- TODO: Fix whatever calls this to use "all", "known", "none" instead of numbers.

]]--
function FlagRSP:getGuildDisp()
	local state;
	local number = -1; -- Defaults to display no guilds.
	
	state = FlagRSPConfigure:GetGuildDisplay();
	
	if ( state == "all" ) then
		number = 1;
	elseif ( state == "known" ) then
		number = 0;
	end
	
	return number;
end


--[[

FlagRSP:plotStatus()

- Gives an overview over all settings.

]]--
function FlagRSP:plotStatus()
	self:setRPFlag(self:getRPFlag());
	self:setCharStatus(self:getCharStatus());
	self:setNameDisp(self:getNameDisp());
	self:setLevelDisp(self:getLevelDisp());
	self:setRankDisp(self:getRankDisp());
	self:setGuildDisp(self:getGuildDisp());
	self:setSurname(self:getOwnSurname());
	self:setTitle(self:getOwnTitle());
end


--[[

FlagRSP:showOwnTooltip()

- Shows the tooltip for the player.

]]--
function FlagRSP:showOwnTooltip()
	--tt = getglobal("FlagRSPTooltip");
	local tt = getglobal("GameTooltip");

	local ttline = getglobal("GameTooltipTextLeft1"):GetText();
   
	if not tt:IsVisible() or string.find(ttline, self.pName) == nil then
		tt:SetOwner(UIParent, "ANCHOR_NONE");
		tt:SetPoint("TOP", "UIParent", "TOP", 0, -32);
      
		tooltipLinesLeft = {};
		tooltipLinesLeft[1] = "";
		tooltipLinesRight = {};
      
		tt:SetBackdropColor(TooltipHandler:getTooltipBGColor("player"));
		TooltipHandler:compileTooltip(tooltipLinesLeft, tooltipLinesRight, TooltipModifierSelf, tt:GetName(), "player");
		tt:Show();
	else
		--tt:Hide();
	end
end


--[[

FlagRSP:printDebug(msg)

- Gives a debug message msg.

]]--
function FlagRSP:printDebug(msg)
	if flagRSP_Debug and msg ~= nil then
		ChatFrame4:AddMessage("|CFF" .. self:getHexString(1*255,2) .. self:getHexString(1*255,2) .. self:getHexString(0.5*255,2) .. "<flagRSP> DEBUG:|r " .. msg,1,1,1);
	end
end


--[[

FlagRSP:print(msg)

- Prints message msg.

]]--
function FlagRSP:print(msg, frame)
	if frame == nil then
		frame = DEFAULT_CHAT_FRAME;
	end

	frame:AddMessage(msg,1,1,0);
end


--[[

FlagRSP:printM(msg)

-- Prints message msg in flagRSP style.

]]--
function FlagRSP:printM(msg)
	DEFAULT_CHAT_FRAME:AddMessage("|CFF" .. self:getHexString(1*255,2) .. self:getHexString(1*255,2) .. self:getHexString(0.5*255,2) .. "<flagRSP>|r " .. msg,1,1,1);
end


--[[

FlagRSP:printE(msg)

-- Prints error message msg in flagRSP style.

]]--
function FlagRSP:printE(msg)
	DEFAULT_CHAT_FRAME:AddMessage("|CFF" .. self:getHexString(1*255,2) .. self:getHexString(1*255,2) .. self:getHexString(0.5*255,2) .. "<flagRSP>|r|CFF" .. self:getHexString(1*255,2) .. self:getHexString(0*255,2) .. self:getHexString(0*255,2) .. "<ERROR>|r" .. msg,1,1,1);
end


--[[

FlagRSP:printA(msg)

-- Prints assertion message msg in flagRSP style.

]]--
function FlagRSP:printA(number)
	self:printM("|Cffff0000ASSERTION #" .. number .. " FAILED!|r Please contact flagRSP developer!");
end


function FlagRSPNewVersionNotificationCheckButton_OnClick()
	local checkState = FlagRSPNewVersionNotificationCheckButton:GetChecked();
	
	if checkState == nil then
		FlagRSPConfigure:SetVersionNotification(true)
	elseif checkState == 1 then
		FlagRSPConfigure:SetVersionNotification(false);
	end
end

function FlagRSPDYKBoxCheckButton_OnClick()
	local checkState = FlagRSPDYKBoxCheckButton:GetChecked();
	
	if checkState == nil then
		FlagRSPConfigure:SetShowTips(true);
	elseif checkState == 1 then
		FlagRSPConfigure:SetShowTips(false);
	end
end

function FlagRSP:showVersionNotification(r, m, n)
	if not UnitAffectingCombat("player") and FlagRSPConfigure.GetVersionNotification and not self.notificationSeen then
		FlagRSPNewVersionNotification:SetWidth(450);
		FlagRSPNewVersionNotification:SetHeight(100);
		FlagRSPNewVersionNotification:Show();
		--FlagRSPNewVersionNotification:SetAlpha(1);
		FlagRSPNewVersionNotification:SetBackdropColor(0,0,0,0.9);

		local text = FlagRSP_Locale_NewVersionText;
      
		text = string.gsub(text, "%%r", r)
		text = string.gsub(text, "%%m", m)
		text = string.gsub(text, "%%n", n)

		FlagRSPNewVersionNotificationTitleText:SetText(FlagRSP_Locale_NewVersionTitle);
		FlagRSPNewVersionNotificationText:SetText(text);
		FlagRSPNewVersionNotificationCheckButtonText:SetText(FlagRSP_Locale_NewVersionCheckButton);
		FlagRSPNewVersionNotificationText:SetWidth(410);

		FlagRSPNewVersionNotificationText:SetTextHeight(14);
		FlagRSPNewVersionNotification:SetScale(2);
		FlagRSPNewVersionNotification:SetScale(1);

		--FlagRSPNewVersionNotificationTitle:SetWidth(FlagRSPNewVersionNotificationTitleText:GetWidth()+40);

		--self:printDebug("flagRSP: a buttons height is: " .. FlagRSPNewVersionNotificationAccept:GetHeight());
      
		FlagRSPNewVersionNotificationCheckButton:SetPoint("BOTTOMLEFT", FlagRSPNewVersionNotification:GetName(), "BOTTOMLEFT", 20, FlagRSPNewVersionNotificationAccept:GetHeight()+20);

		FlagRSPNewVersionNotification:SetPoint("TOP", "UIParent", "TOP", 0, -150);
		FlagRSPNewVersionNotification:SetPoint("BOTTOM", "UIParent", "TOP", 0, -220-FlagRSPNewVersionNotificationText:GetHeight()-FlagRSPNewVersionNotificationAccept:GetHeight()-FlagRSPNewVersionNotificationCheckButton:GetHeight());
	
		--FlagRSPDescEditor:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 10, -600);

		self.notificationSeen = true;
	end
end


function FlagRSPDescWarnBoxAccept_OnClick()
	--PlaySound("gsTitleOptionOK");
	--FRIENDLISTStaticPopUpEdit_EditBoxOnEnterPressed();
	FlagRSPDescWarnBox:Hide();
	--FlagRSPDescEditor:Show();
	--FlagRSPDescEditorEditBox:Show();
	--FlagRSPDescEditor:SetBackdropColor(0,0,0,0.3);
	--FlagRSPDescEditorEditBox:SetText(FlagRSPSettings[self.rName][self.pName].CharDesc);

	--FlagRSPDescEditor:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 10, -150);
   
	if ( FlagRSPDescWarnBox.warning == "description" ) then
		FriendlistFrame.descBoxWarningSeen = true;
	else
		FriendlistFrame.nameBoxWarningSeen = true;
	end
end

function FlagRSP:showEditBox(message)
	FlagRSPDescWarnBox.warning = "givenname";
	if ( message == nil ) then
		message = FlagRSP_Locale_DescWarnText;
		FlagRSPDescWarnBox.warning = "description";
	end
	
	FlagRSPDescWarnBox:SetWidth(550);
	FlagRSPDescWarnBox:SetHeight(100);
	FlagRSPDescWarnBox:Show();
	--FlagRSPDescWarnBox:SetAlpha(1);
	FlagRSPDescWarnBox:SetBackdropColor(0,0,0,0.9);
	
	FlagRSPDescWarnBoxTitleText:SetText(FlagRSP_Locale_DescWarnTitle);
	FlagRSPDescWarnBoxText:SetText(message);
	FlagRSPDescWarnBoxText:SetWidth(510);

	FlagRSPDescWarnBoxText:SetTextHeight(12);
	FlagRSPDescWarnBox:SetScale(2);
	FlagRSPDescWarnBox:SetScale(1);

	--self:printDebug("flagRSP: a buttons height is: " .. FlagRSPDescWarnBoxAccept:GetHeight());

	FlagRSPDescWarnBox:SetPoint("TOP", "UIParent", "TOP", 0, -150);
	FlagRSPDescWarnBox:SetPoint("BOTTOM", "UIParent", "TOP", 0, -220-FlagRSPDescWarnBoxText:GetHeight()-FlagRSPDescWarnBoxAccept:GetHeight());

	--FlagRSPDescEditor:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 10, -600);
end


function FlagRSP:showDYKBox(nextTip)
	if nextTip == nil then
		nextTip = false;
	end

	--self:printM("test");

	FlagRSPDYKBox:SetWidth(550);
	FlagRSPDYKBox:SetHeight(100);
	FlagRSPDYKBox:Show();
	--FlagRSPDYKBox:SetAlpha(1);
	FlagRSPDYKBox:SetBackdropColor(0,0,0,0.9);
   
	if FlagRSPConfigure:GetShowTips() then
		FlagRSPDYKBoxCheckButton:SetChecked(0);
	else
		FlagRSPDYKBoxCheckButton:SetChecked(1);
	end
   
	local text = FlagRSP_Locale_TipText[math.random(1,table.getn(FlagRSP_Locale_TipText))];

	--self:printM(text);

	FlagRSPDYKBoxCheckButtonText:SetText(FlagRSP_Locale_DYKCheckText);
   
	FlagRSPDYKBoxTitleText:SetText(FlagRSP_Locale_DYKTitle);
	--TooltipHandler:ParseString()
	FlagRSPDYKBoxText:SetText(text);
	FlagRSPDYKBoxText:SetWidth(510);
   
	FlagRSPDYKBoxAccept:Hide();  
   
	FlagRSPDYKBoxText:SetTextHeight(12);
	FlagRSPDYKBox:SetScale(2);
	FlagRSPDYKBox:SetScale(1);
   
	--self:printDebug("flagRSP: a buttons height is: " .. FlagRSPDYKBoxAccept:GetHeight());
   
	FlagRSPDYKBoxCheckButton:SetPoint("BOTTOMLEFT", "FlagRSPDYKBox", "BOTTOMLEFT", 20, FlagRSPDYKBoxClose:GetHeight()+20); 
   
	FlagRSPDYKBox:SetPoint("TOP", "UIParent", "TOP", 0, -150);
	FlagRSPDYKBox:SetPoint("BOTTOM", "UIParent", "TOP", 0, -220-FlagRSPDYKBoxText:GetHeight()-FlagRSPDYKBoxAccept:GetHeight()- FlagRSPDYKBoxCheckButton:GetHeight());
   
end



function FlagRSP:showWelcomeBox(full)
	FlagRSPWelcomeBox:SetWidth(550);
	FlagRSPWelcomeBox:SetHeight(100);
	FlagRSPWelcomeBox:Show();
	--FlagRSPWelcomeBox:SetAlpha(1);
	FlagRSPWelcomeBox:SetBackdropColor(0,0,0,0.9);
   
	local text
	if full then
		text = TooltipHandler:ParseString(FlagRSP_Locale_WelcomeHeader .. FlagRSP_Locale_WelcomeText .. FlagRSP_Locale_WelcomeHomepage ..  FlagRSP_Locale_WelcomeChanges);
	else
		text = TooltipHandler:ParseString(FlagRSP_Locale_WelcomeHeader .. FlagRSP_Locale_WelcomeHomepage .. FlagRSP_Locale_WelcomeChanges);
	end

	FlagRSPWelcomeBoxTitleText:SetText(FlagRSP_Locale_WelcomeTitle);
	--TooltipHandler:ParseString()
	FlagRSPWelcomeBoxText:SetText(text);
	FlagRSPWelcomeBoxText:SetWidth(510);

	FlagRSPWelcomeBoxText:SetTextHeight(12);
	FlagRSPWelcomeBox:SetScale(2);
	FlagRSPWelcomeBox:SetScale(1);

	--self:printDebug("flagRSP: a buttons height is: " .. FlagRSPWelcomeBoxAccept:GetHeight());

	FlagRSPWelcomeBox:SetPoint("TOP", "UIParent", "TOP", 0, -150);
	FlagRSPWelcomeBox:SetPoint("BOTTOM", "UIParent", "TOP", 0, -220-FlagRSPWelcomeBoxText:GetHeight()-FlagRSPWelcomeBoxAccept:GetHeight());

	--FlagRSPDescEditor:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 10, -600);
end


function FlagRSPWelcomeBoxAccept_OnClick()
	FlagRSPWelcomeBox:Hide();

	if FlagRSPConfigure:GetShowTips() then
		FlagRSP:showDYKBox();
	end
end

--[[

FlagRSP:OnLoad(frame)

-- A unified version of the OnLoad from the Friendlist and FlagRSP.

]]--
function FlagRSP:OnLoad(frame)
	frame:RegisterEvent("PLAYER_ENTERING_WORLD");
	frame:RegisterEvent("ADDON_LOADED");
	frame:RegisterEvent("ZONE_CHANGED_NEW_AREA");

	frame:RegisterEvent("ADDON_ACTION_FORBIDDEN");
	frame:RegisterEvent("ADDON_ACTION_BLOCKED");
	
	frame:RegisterEvent("CHAT_MSG_SYSTEM");
	frame:RegisterEvent("CHAT_MSG_CHANNEL");
	
	frame:RegisterEvent("VARIABLES_LOADED");  -- Fires when saved variables are loaded

	frame:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	frame:RegisterEvent("PLAYER_TARGET_CHANGED");

	--SlashCmdList["RSPAFK"] = self:toggleAFK();
	--SLASH_RSPAFK1 = "/afk";

	SlashCmdList["RSP"] = xTooltip_Msg;
	SLASH_RSP1 = "/rsp";
	
	SlashCmdList["RSPAN"] = function () FlagRSP:enableRPTags(); end
	SLASH_RSPAN1 = "/rspan";
	SLASH_RSPAN2 = "/rspon";
	
	SlashCmdList["RSPAUS"] = function () FlagRSP:disableRPTags(); end
	SLASH_RSPAUS1 = "/rspaus";
	SLASH_RSPAUS2 = "/rspoff";

	SlashCmdList["RPHELP"] = function () FlagRSP:sendRPTicket(); end
	SLASH_RPHELP1 = "/rpticket";
	SLASH_RPHELP2 = "/rphelp";

	SlashCmdList["RELOADUI"] = ReloadUI;
	SLASH_RELOADUI1 = "/rui";
	SLASH_RELOADUI2 = "/reloadui";
	
	SlashCmdList["FRIENDLISTCOMMAND"] = FRIENDLIST_SlashHandler;
	SLASH_FRIENDLISTCOMMAND1 = "/fl";
	SLASH_FRIENDLISTCOMMAND2 = "/friendlist";
	
	SlashCmdList["ADD"] = addfriend_Msg;
	SLASH_ADD1 = "/add";
	
	--GameTooltip:SetScript("OnTooltipSetUnit", function() FlagRSP:BuildTooltip() end);

	FlagRSP_Version = self.versionString;
	self.VERSION = FlagRSP_Version;
end

--[[

- FlagRSP:OnEvent(event, ...)

-- This will handle all the event handlers between the Friendlist and
   FlagRSP.  It's a more unified approach than the previous ones.

]]--
function FlagRSP:OnEvent(event, ...)
	if event == "PLAYER_ENTERING_WORLD" or event == "ADDON_LOADED" then
		if not self.playerEnteredWorld then

			self.pName, self.rName = FlagRSPConfigure:Initialize();
			FlagHandler:SetRealm(self.rName);

			self.newTickPost = GetTime() + 60;
			self.newTickPostHigh = GetTime() + 50;

			local t = 0;
			while FlagRSP_Locale_OnLoad[t] ~= nil do
				self:printM(TooltipHandler:ParseString(FlagRSP_Locale_OnLoad[t]),1.0, 1.0, 0.0);
				t = t + 1;
			end

			self.sessionStartTick = GetTime();
		 
			--table.insert(UnitPopupMenus["PLAYER"], table.getn(UnitPopupMenus["PLAYER"]), "FLAGRSP_ADDEDIT");
			--table.insert(UnitPopupMenus["PLAYER"], table.getn(UnitPopupMenus["PLAYER"]), "FLAGRSP_TOGGLEINFOBOX");

			--UnitPopupButtons["FLAGRSP_ADDEDIT"] = { text = "Add/Edit flagRSP entry", dist = 0 };

			if ( FriendlistData == nil and ( friendData == nil or friendData[self.rName] == nil or friendData[self.rName][self.pName] == nil ) ) then
				Friendlist:InitializeData();
			elseif ( friendData ~= nil and FriendlistData == nil ) then
				Friendlist:ConvertToPerCharacterFriendlist();
			end
			
			if ( FlagRSPData == nil and ( FlagRSPSettings == nil or FlagRSPSettings[self.rName] == nil or FlagRSPSettings[self.rName][self.pName] == nil ) ) then
				FlagRSPConfigure:initializeSettings();
			elseif ( FlagRSPData == nil and FlagRSPSettings[self.rName][self.pName] ~= nil ) then
				FlagRSPConfigure:ConvertToPerCharacterSettings();
			end
			
			if FlagRSPConfigure:GetStandby() then
				self:printM(FlagRSP_Locale_StandBy);
				self:printM(FlagRSP_Locale_StandBy2);
				self:LeaveChannel();
				FlagRSPConfigure:SetStandby(true);
				self.standby = true;
			end
			
			TooltipHandler:buildSkinnableTable();
			FlagRSPInfo:purgeBoxList();
			FlagHandler:purgeFlags();
			
--~ 			if ( FlagRSPConfigure:DisplayWelcomeBox?() == "newuser" ) then
--~ 				self:showWelcomeBox(true)
--~ 			elseif ( FlagRSPConfigure:DisplayWelcomeBox?() == "newversion" ) then
--~ 				self:showWelcomeBox(false)
--~ 			end

			if ( FlagRSPConfigure:GetShowTips() ) then
				self:showDYKBox()
			end

			self.playerEnteredWorld = true;
			FlagRSPConfigure:SetButtonPosition(FlagRSPConfigure:GetButtonPosition());
		end

		if myAddOnsFrame_Register then
			myAddOnsFrame_Register(flagRSPDetails, flagRSPHelp);
		end
	elseif (event == "VARIABLES_LOADED") or (event == "UNIT_NAME_UPDATE" and select(1, ...) == 'player') then
		if not FlagRSPConfigure:IsInitialized() then
			return;
		end

		FRIENDLIST_Update();
		
		if ( friendData ~= nil and FriendlistData == nil ) then
			Friendlist:ConvertToPerCharacterFriendlist();
		elseif ( friendData == nil and FriendlistData == nil ) then
			Friendlist:InitializeData();
		end
		
		if ( FlagRSPSettings[self.rName][self.pName] ~= nil and FlagRSPData == nil ) then
			FlagRSPConfigure:ConvertToPerCharacterSettings();
		elseif ( FlagRSPSettings[self.rName][self.pName] == nil and FlagRSPData == nil ) then
			FlagRSPConfigure:initializeSettings();
		end
		
		FRIENDLISTMiniMap_OnLoad();
	elseif event == "CHAT_MSG_SYSTEM" then
		if ( self:isAFKEvent() ) then
			if not self.collecting then
				flagRSP_AFK = true;
				self:printM(FlagRSP_Locale_AFK_Activated);
			else
			end
	 
			flagRSP_AFKPosx, flagRSP_AFKPosy = GetPlayerMapPosition("player");
			self:printDebug("flagRSP: DEBUG: AFK: x_position: " .. flagRSP_AFKPosx .. " y_position: " .. flagRSP_AFKPosy);
		end
		if ( self:isnotAFKEvent() ) then
			flagRSP_AFK = false;
			self:printM(FlagRSP_Locale_AFK_Deactivated);
		end
		--self:checkAFK(arg2);

	elseif event == "CHAT_MSG_CHANNEL" then
		--self:print(arg9);
		--self:checkAFK(arg2);

		if string.lower(select(9, ...)) == string.lower(xTP_ChannelName) then
			--self:printDebug(arg9);
			--self:printDebug(arg4);

			self:caughtChatMessage(select(1, ...), select(2, ...));
			self:chatMessageHandler(select(1, ...), select(2, ...));	 
		end
	elseif event == "UPDATE_MOUSEOVER_UNIT" then
		self:UpdateTooltip()
		
		Friendlist:updateEntry("mouseover", "unitID");

		--Grab the name of who we've moused-over
		local name = UnitName("mouseover");
		local guildName = GetGuildInfo("mouseover");

		--If they are on our list, update the tooltip and play a sound
		if (Friendlist:isFriend(name)) then
			if (Friendlist:GetFriendstate(name) <= -10) then
				if (FlagRSPConfigure:GetSoundsSetting() == true) then
					if GetTime() > FriendlistFrame.soundTick then 
						--self:printM("Hey!");
						PlaySound("igMiniMapZoomIn");
						FriendlistFrame.soundTick = GetTime() + 0.5; 
					end	       
				end
			end
		elseif (Friendlist:isFriend(guildName)) then
			if (Friendlist:GetFriendstate(guildName) <= -10) then
				if (FlagRSPConfigure:GetSoundsSetting() == true) then
	       
					if GetTime() > FriendlistFrame.soundTick then 
						--self:printM("Hey!");
						PlaySound("igMiniMapZoomIn");
						FriendlistFrame.soundTick = GetTime() + 0.5; 
					end
				end
			end
		end
	elseif ( event == "ADDON_ACTION_FORBIDDEN" ) then
		if ( select(1, ...) ~= nil and select(2, ...) ~= nil ) then
			if ( select(1, ...) == "flagRSP" or select(1, ...) == "flagRSPLoader" or select(1, ...) == "FriendList" ) then
--~ 				ChatFrame1:AddMessage("ADDON_ACTION_FORBIDDEN fired.  " .. select(1, ...) .. " attempted to call " .. select(2, ...));
				local debugMsg;
				debugMsg = "Call stack was: " .. debugstack(3, 2, 3);
--				ChatFrame1:AddMessage("ADDON_ACTION_FORBIDDEN fired.  " .. select(1, ...) .. " attempted to call " .. select(2, ...) .. " when it was protected.");
--				ChatFrame1:AddMessage(debugMsg);
			end
		end
	elseif ( event == "ADDON_ACTION_BLOCKED" ) then
		if ( select(1, ...) ~= nil and select(2, ...) ~= nil ) then
			if ( select(1, ...) == "flagRSP" or select(1, ...) == "flagRSPLoader" or select(1, ...) == "FriendList" ) then
				local debugMsg;
				debugMsg = "Call stack was: " .. debugstack(3, 2, 3);
--				ChatFrame1:AddMessage("ADDON_ACTION_BLOCKED fired.  " .. select(1, ...) .. " attempted to call " .. select(2, ...) .. " when it was protected.");
--				ChatFrame1:AddMessage(debugMsg);
			end
		end
	end
	if ( event == "PLAYER_TARGET_CHANGED" ) then
		local targetName = UnitName("target");
		if Friendlist:isFriend(targetName) then
			--UnitPopupButtons["FLAGRSP_ADDEDIT"] = { text = TooltipHandler:ParseString(FlagRSP_Locale_EditEntry, "target"), dist = 0 };
		else
			--UnitPopupButtons["FLAGRSP_ADDEDIT"] = { text = TooltipHandler:ParseString(FlagRSP_Locale_AddEntry, "target"), dist = 0 };
		end

		--UnitPopupButtons["FLAGRSP_TOGGLEINFOBOX"] = { text = TooltipHandler:ParseString(FlagRSP_Locale_ToggleBox, "target"), dist = 0 };

		--self:printDebug("Target changed!");
		FlagRSPInfo.targetChanged = true;
      
		--FlagRSPInfoBoxButton:SetPoint("CENTER", "TargetFrame", "CENTER", 65, 30);
		--FlagRSPInfoBoxButtonNormalTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Elite");

		-- flokru: I linked hiding of levels in Targetframe to level changing in tooltips.
		if ( FlagRSPConfigure:isInitialized() and FlagRSPConfigure:GetLevelDisplay() == "relative" ) then
			local targetLevel = UnitLevel("target");
			local targetLevelText = "";
			
			if targetLevel > 0 then
				Diff = targetLevel - UnitLevel("player");

				if UnitClassification("target") == "elite" then
					Diff = Diff + 5;
				end

				local r,g,b;
                
				if ( level == -1 or Diff >= 7 ) then
					r = QuestDifficultyColor["impossible"].r;
					g = QuestDifficultyColor["impossible"].g;
					b = QuestDifficultyColor["impossible"].b;
				elseif ( Diff >= 4 ) then
					r = QuestDifficultyColor["verydifficult"].r;
					g = QuestDifficultyColor["verydifficult"].g;
					b = QuestDifficultyColor["verydifficult"].b;
				elseif ( Diff >= -1 ) then
					r = QuestDifficultyColor["difficult"].r;
					g = QuestDifficultyColor["difficult"].g;
					b = QuestDifficultyColor["difficult"].b;
				elseif ( -Diff <= GetQuestGreenRange() ) then
					r = QuestDifficultyColor["standard"].r;
					g = QuestDifficultyColor["standard"].g;
					b = QuestDifficultyColor["standard"].b;
				else
					r = QuestDifficultyColor["trivial"].r;
					g = QuestDifficultyColor["trivial"].g;
					b = QuestDifficultyColor["trivial"].b;
				end
                
				TargetLevelText:SetTextColor(r,g,b);
                
				if ( FlagRSPConfigure:GetLevelDisplay() == "relative" ) then
					if ( level == -1 or Diff >= 10 ) then
						targetLevelText = FlagRSP_Locale_TF_Impossible;
					elseif ( Diff >= 7 ) then
						targetLevelText = FlagRSP_Locale_TF_Estrong;
					elseif ( Diff >= 4 ) then
						targetLevelText = FlagRSP_Locale_TF_Vstrong;
					elseif ( Diff >= 2 ) then
						targetLevelText = FlagRSP_Locale_TF_Strong;
					elseif ( Diff >= -1 ) then
						targetLevelText = FlagRSP_Locale_TF_Equal;
					elseif ( Diff >= -GetQuestGreenRange()) then
						targetLevelText = FlagRSP_Locale_TF_Weak;
					elseif ( Diff >= (-4 -GetQuestGreenRange())) then
						targetLevelText = FlagRSP_Locale_TF_Puny;
					else
						targetLevelText = FlagRSP_Locale_TF_Epuny;
					end
				end

				TargetLevelText:SetText(targetLevelText);
			end
		end
		
		-- Make sure target frame is back to original color
		TargetFrameTexture:SetVertexColor(1.0, 1.0, 1.0, TargetFrameTexture:GetAlpha());
      
		Friendlist:updateEntry("target","unitID");

		if UnitExists("target") and UnitPlayerControlled("target") then
			local targetName = UnitName("target");
			local guildName = GetGuildInfo("target");
			local r, g, b, t;
			if (Friendlist:isFriend(targetName)) then
				r,g,b,t = Friendlist:GetFriendstateInfo(Friendlist:GetFriendstate(targetName));
				TargetFrameTexture:SetVertexColor(r, g, b, TargetFrameTexture:GetAlpha());
				TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Elite");
			elseif (Friendlist:isFriend(guildName)) then
				r,g,b,t = Friendlist:GetFriendstateGuildInfo(Friendlist:GetFriendstate(guildName));
				TargetFrameTexture:SetVertexColor(r, g, b, TargetFrameTexture:GetAlpha());
				TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame");
			else
				-- Not FRIEND target, make sure Texture Overlay is back to original color
				if UnitClassification("target") == "elite" then
--					TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Elite");	       
				else 
--					TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame");
				end
--				TargetFrameTexture:SetVertexColor(1.0, 1.0, 1.0, TargetFrameTexture:GetAlpha());
			end
		end
		--FRIENDLIST_Update();

		--self:print(Friendlist:GetNotes(UnitName("target")));
		FlagRSPInfo:displayBox("target");
	end
end

function flagRSP_toggleHideNames()
	if not self:getNameDisp() then
		self:setNameDisp(true);
	else
		self:setNameDisp(false); 
	end
end

function flagRSP_toggleLevelDisp()
	local display = self:getLevelDisp();
	
	if ( display == "none" ) then
		self:setLevelDisp("relative");
	elseif ( display == "relative" ) then
		self:setLevelDisp("color");
	else
		self:setLevelDisp("none");
	end
end


function FlagRSP:setNameDisp(value)
	local mesg;

	if value == 0 then value = false; end
	if value == 1 then value = true; end

	if value ~= true and value ~= false then return; end

	if value then
		mesg = FlagRSP_Locale_HideNames;
		FlagRSPConfigure:SetNameDisplay(true);
	else
		mesg = FlagRSP_Locale_UnhideNames;
		FlagRSPConfigure:SetNameDisplay(false);
	end

	self:printM(mesg);
end


function FlagRSP:getNameDisp()
	return FlagRSPConfigure:GetNameDisplay();
end


function FlagRSP:setLevelDisp(value)
	local mesg;

	if ( value == "none" ) then
		mesg = FlagRSP_Locale_HideLevels;
		FlagRSPConfigure:SetLevelDisplay("relative");
	elseif ( value == "relative" ) then
		mesg = FLagRSP_Locale_ColoredLevels;
		FlagRSPConfigure:SetLevelDisplay("color");
	else
		mesg = FlagRSP_Locale_UnhideLevels;
		FlagRSPConfigure:SetLevelDisplay("none");
	end

	self:printM(mesg);
end

function FlagRSP:getLevelDisp()
	return FlagRSPConfigure:GetLevelDisplay();
end


function FlagRSP:setRPFlag(RPType)
	local mesg;
	local changed = false;

	if RPType < 0 or RPType > 5 then return; end

	if RPType ~= FlagRSPConfigure:GetRPFlag() then
		if RPType == 0 then
			-- flokru: Now we save roleplaying behaviour before unsetting rp-flag.
			if FlagRSPConfigure:GetRPFlag() ~= 0 then
				FlagRSPConfigure:SetOldRPFlag(FlagRSPConfigure:GetRPFlag());
			end
			mesg = FlagRSP_Locale_NoRPFlagSet;
		elseif RPType == 1 then
			mesg = FlagRSP_Locale_NormalRPFlagSet;
		elseif RPType == 2 then
			mesg = FlagRSP_Locale_CasualFlagSet;
		elseif RPType == 3 then
			mesg = FlagRSP_Locale_FulltimeFlagSet;
		elseif RPType == 4 then
			mesg = FlagRSP_Locale_BeginnerFlagSet;
		elseif RPType == 5 then
			-- flokru: I love recursion :)
			if FlagRSPConfigure:GetOldRPFlag() ~= nil then
				self:setRPFlag(FlagRSPConfigure:GetOldRPFlag());
			else
				self:setRPFlag(1);
			end
			return;
		end
      
		self:printM(mesg);
		FlagRSPConfigure:SetRPFlag(RPType);

		changed = true;
	end

	return changed;
end


function xTooltip_Msg(msg)

	if string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_HideNames_Cmd))) == FlagRSP_Locale_HideNames_Cmd then
		flagRSP_toggleHideNames();
	elseif string.lower(string.sub(msg, 1, 4)) == "stat" then      
		FlagRSP:plotStats();
	elseif string.lower(string.sub(msg, 1, 7)) == "collect" then      
		if FlagRSP.collecting then
			FlagRSP:printM(FlagRSP_Locale_NoCollecting);
			FlagRSP.collecting = false;
		else
			FlagRSP:printM(FlagRSP_Locale_Collecting);
			FlagRSP.collecting = true;
		end
	elseif string.lower(string.sub(msg, 1, 7)) == "welcome" then      
		FlagRSP:showWelcomeBox();
	elseif string.lower(string.sub(msg, 1, 4)) == "tips" then      
		FlagRSP:showDYKBox();
	elseif string.lower(msg) == "standby" then
		if FlagRSPConfigure:GetStandby() then
			FlagRSP:printM(FlagRSP_Locale_NoStandBy);
			FlagRSP:JoinChannel();
			FlagRSPConfigure:SetStandby(false);
			FlagRSP.standby = false;
		else
			FlagRSP:printM(FlagRSP_Locale_StandBy);
			FlagRSP:printM(FlagRSP_Locale_StandBy2);
			FlagRSP_LeaveChannel();
			FlagRSPConfigure:SetStandby(true);
			FlagRSP.standby = true;
		end
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_LevelDisp_Cmd))) == FlagRSP_Locale_LevelDisp_Cmd then
		flagRSP_toggleLevelDisp();
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_RankDisp_Cmd))) == FlagRSP_Locale_RankDisp_Cmd then
		FlagRSP:toggleRankDisp();
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_GuildDisp_Cmd))) == FlagRSP_Locale_GuildDisp_Cmd then
		FlagRSP:toggleGuildDisp();
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_Beginner_Cmd))) == FlagRSP_Locale_Beginner_Cmd then
		FlagRSP:setRPFlag(4);
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_Casual_Cmd))) == FlagRSP_Locale_Casual_Cmd then
		FlagRSP:setRPFlag(2);
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_Normal_Cmd))) == FlagRSP_Locale_Normal_Cmd then
		FlagRSP:setRPFlag(1);
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_Fulltime_Cmd))) == FlagRSP_Locale_Fulltime_Cmd then
		FlagRSP:setRPFlag(3);
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_NoRP_Cmd))) == FlagRSP_Locale_NoRP_Cmd then
		FlagRSP:setRPFlag(0);
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_OOC_Cmd))) == FlagRSP_Locale_OOC_Cmd then
		FlagRSP:setCharStatus("ooc");
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_IC_Cmd))) == FlagRSP_Locale_IC_Cmd then
		FlagRSP:setCharStatus("ic");
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_ICFFA_Cmd))) == FlagRSP_Locale_ICFFA_Cmd then
		FlagRSP:setCharStatus("ffa-ic");
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_ST_Cmd))) == FlagRSP_Locale_ST_Cmd then
		FlagRSP:setCharStatus("st");
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_NoCStatus_Cmd))) == FlagRSP_Locale_NoCStatus_Cmd then
		FlagRSP:setCharStatus("none");
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_Status_Cmd))) == FlagRSP_Locale_Status_Cmd then
		FlagRSP:plotStatus();
	elseif string.lower(string.sub(msg, 1, 7)) == "postlow" then      
		FlagRSP:PostFlags();
	elseif string.lower(string.sub(msg, 1, 8)) == "posthigh" then      
		FlagRSP:PostFlags();
	elseif string.lower(string.sub(msg, 1, 4)) == "post" then      
		FlagRSP:PostFlags();
	elseif string.lower(msg) == "toggletooltip" then
		if ( FlagRSPConfigure:GetTooltipStyle() == "normal" ) then
			FlagRSP:printM(FlagRSP_Locale_LightModifyTooltip);
			FlagRSPConfigure:SetTooltipStyle("light");
		elseif ( FlagRSPConfigure:GetTooltipStyle() == "light" ) then
			FlagRSP:printM(FlagRSP_Locale_NoModifyTooltip);
			FlagRSPConfigure:SetTooltipStyle("none");
		elseif ( FlagRSPConfigure:GetTooltipStyle() == "none" ) then
			FlagRSP:printM(FlagRSP_Locale_ModifyTooltip);
			FlagRSPConfigure:SetTooltipStyle("normal");
		end
	elseif string.lower(string.sub(msg, 1, string.len("purgeinterval"))) == "purgeinterval" then
		FlagRSP:printDebug("new purge interval: " .. string.sub(msg, string.len("purgeinterval")+2, string.len(msg)));
		if string.len(msg) > string.len("purgeinterval")+1 then
			local n = tonumber(string.sub(msg, string.len("purgeinterval")+2, string.len(msg)));
			if n ~= nil then
				FlagRSPSettings.FlagPurgeInterval = n*86400;
				FlagRSP:printDebug("new purge interval (set): " .. FlagRSPSettings.FlagPurgeInterval);
				local msg = string.gsub(FlagRSP_Locale_PurgeInterval, "%%s", n);
				FlagRSP:printM(msg);
			end
		else
			local msg = string.gsub(FlagRSP_Locale_PurgeInterval, "%%s", math.floor(FlagRSPSettings.FlagPurgeInterval/86400*10)/10);
			FlagRSP:printM(msg);
		end	 
	elseif string.lower(msg) == "purge" then
		FlagHandler:purgeAllFlags();
	elseif string.lower(string.sub(msg, 1, 6)) == "toggle" then
		if FlagRSP:getRPFlag() == 0 then
			FlagRSP:setRPFlag(5);
		else
			FlagRSP:setRPFlag(0);
		end
		FlagRSP:PostFlags();
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_Surname_Cmd))) == FlagRSP_Locale_Surname_Cmd then
		FlagRSP:setSurname(string.sub(msg, string.len(FlagRSP_Locale_Surname_Cmd)+2, string.len(msg)));
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_Title_Cmd))) == FlagRSP_Locale_Title_Cmd then
		FlagRSP:setTitle(string.sub(msg, string.len(FlagRSP_Locale_Title_Cmd)+2, string.len(msg)));
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_AFK_Cmd))) == FlagRSP_Locale_AFK_Cmd then
		SendChatMessage(string.sub(msg, string.len(FlagRSP_Locale_AFK_Cmd)+2, string.len(msg)), "AFK");
		--FlagRSP:toggleAFK(string.sub(msg, string.len(FlagRSP_Locale_AFK_Cmd)+2, string.len(msg)), true);
	elseif string.lower(string.sub(msg, 1, string.len("edit"))) == "edit" then
		FRIENDLIST_ToggleGUI();
		Friendlist_TabHandler(2);
	elseif string.lower(string.sub(msg, 1, string.len("tmulti"))) == "tmulti" then
		FlagRSP:chatMessageHandler("<RP>53.2332<CS3><DV>A2<W>Ooops<V>0.5.0", "Gorrmar")
	elseif string.lower(string.sub(msg, 1, string.len("uinames"))) == "uinames" then
		FlagRSP:printDebug(GetCVar("UnitNamePlayer"));
		if GetCVar("UnitNamePlayer") == "1" then
			FlagRSP:printM(FlagRSP_Locale_UINamesDisabled);
			SetCVar("UnitNamePlayer","0");
		elseif GetCVar("UnitNamePlayer") == "0" then
			FlagRSP:printM(FlagRSP_Locale_UINamesEnabled);
			SetCVar("UnitNamePlayer","1");
		end
	elseif string.lower(string.sub(msg, 1, string.len("alwaysinfobox"))) == "alwaysinfobox" then
		if FlagRSPConfigure:GetInfoBoxState() then
			FlagRSP:printM(FlagRSP_Locale_InfoBoxTradBehaviour);
			FlagRSPConfigure:SetInfoBoxState(false);
		else 
			FlagRSP:printM(FlagRSP_Locale_AlwaysShowInfoBox);
			FlagRSPConfigure:SetInfoBoxState(true);
		end
	elseif string.lower(string.sub(msg, 1, string.len("resetbutton"))) == "resetbutton" then
		FlagRSPConfigure:SetInfoBoxButtonCoords(-1, -1);
		--FlagRSPInfoBoxButton:SetPoint("CENTER", "TargetFrame", "CENTER", 65, 30);
	elseif string.lower(string.sub(msg, 1, string.len(FlagRSP_Locale_OwnTooltip_Cmd))) == FlagRSP_Locale_OwnTooltip_Cmd then
		if not FlagRSP.displayOwnTooltip then
			FlagRSP.displayOwnTooltip = true;
			FlagRSP:showOwnTooltip();
		else
			FlagRSP.displayOwnTooltip = false;	 
		end
	elseif string.lower(string.sub(msg, 1, 5)) == "debug" then
		if flagRSP_Debug then
			FlagRSP:printM("Left debug mode.",1.0, 1.0, 0.0);
			flagRSP_Debug = false;
		else
			FlagRSP:printM("Entered debug mode.",1.0, 1.0, 0.0);
			flagRSP_Debug = true;
		end
	-- TODO: Add in Locale strings for these options.
	elseif ( string.lower(string.sub(msg, 1, string.len("coloredlevels"))) == "coloredlevels" ) then
		if ( FlagRSPSettings[FlagRSP.rName][FlagRSP.pName].colorLevelText ) then
			FlagRSPSettings[FlagRSP.rName][FlagRSP.pName].colorLevelText = false;
			FlagRSP.printM("colorLevelText set to false");
		else
			FlagRSPSettings[FlagRSP.rName][FlagRSP.pName].colorLevelText = true;
			FlagRSP.printM("colorLevelText set to true");
		end
	elseif ( string.lower(string.sub(msg, 1, string.len("classcolor"))) == "classcolor" ) then
		if ( FlagRSPConfigure:GetClassColor() ) then
			FlagRSPConfigure:SetClassColor(false);
			FlagRSP:printM("colorClassText set to false");
		else
			FlagRSPConfigure:SetClassColor(true);
			FlagRSP:printM("colorClassText set to true");
		end
	elseif ( string.lower(string.sub(msg, 1, string.len("guildrank"))) == "guildrank" ) then
		if ( FlagRSPConfigure:GetGuildRank() ) then
			FlagRSPConfigure:SetGuildRank(false);
			FlagRSP:printM("showGuildRank set to false");
		else
			FlagRSPConfigure:SetGuildRank(true);
			FlagRSP:printM("showGuildRank set to true");
		end
	else
		local t = 0;
		while FlagRSP_Locale_Help[t] ~= nil do
			DEFAULT_CHAT_FRAME:AddMessage(FlagRSP_Locale_Help[t],1.0, 1.0, 1.0);
			t = t + 1;
		end
	end
end


function FlagRSP:JoinChannel()
	local id = GetChannelName(xTP_ChannelName);

	--DEFAULT_CHAT_FRAME:AddMessage("DEBUG: JoinChannel, id is:" .. id);

	if not (id >= 1) then 
		self:printM(FlagRSP_Locale_ReJoinedChannel);
		xTP_ChannelName = "xtensionxtooltip2";
		JoinPermanentChannel(xTP_ChannelName, "", DEFAULT_CHAT_FRAME:GetID(), 1);
		id = GetChannelName(xTP_ChannelName);
	end
end

function FlagRSP:LeaveChannel()
	xTP_ChannelName = "xtensionxtooltip2";
	LeaveChannelByName(xTP_ChannelName);
end

--[[

- FlagRSP:UpdateTooltip()

-- Updates the tooltip with the roleplaying data.

]]--

function FlagRSP:UpdateTooltip()
	-- Check the type of style the user has set.
	local tooltipBase = "GameTooltip";	
	local tooltipUnit = "mouseover";
	
	if ( FlagRSPConfigure:GetTooltipStyle() == "normal" ) then
		-- Perform the standard tearing down and rebuilding of the tooltip.
		
		--Uncomment this in 2.1.
		--_, tooltipUnit = tooltipBase:GetUnit();
		
		GameTooltip:SetBackdropColor(TooltipHandler:getTooltipBGColor(tooltipUnit));
		
		local modifier = "";
		if ( UnitIsPlayer(tooltipUnit) ) then
			if ( not self:getNameDisp() or TooltipHandler:playerIsKnown(UnitName(tooltipUnit)) ) then
				modifier = "known";
			elseif ( self:playerIsIgnored(UnitName(tooltipUnit)) ) then
				modifier = "ignored";
				getglobal(tooltipBase):SetBackdropColor(1,0,0);
			else
				modifier = "unknown";
			end
		else
			if ( TooltipHandler:isPet(tooltipUnit) ) then
				modifier = "pet";
			else
				modifier = "npc";
				TooltipHandler:ParseNPCTooltip();
			end
		end
		
		TooltipHandler:BuildTooltip(modifier, tooltipBase, tooltipUnit);
	elseif ( FlagRSPConfigure:GetTooltipStyle() == "light" ) then
		-- This is only a lightly modified tooltip.
		if UnitIsPlayer(tooltipUnit) or UnitPlayerControlled(tooltipUnit) then
			--self:printDebug("light modify");
			
			local name;
			if UnitPVPRank(tooltipUnit) ~= nil and GetPVPRankInfo(UnitPVPRank(tooltipUnit), tooltipUnit) ~= nil then
				name = GetPVPRankInfo(UnitPVPRank(tooltipUnit), tooltipUnit) .. " " .. UnitName(tooltipUnit);
			else
				name = UnitName(tooltipUnit);
			end
			local surname = TooltipHandler:getSurname(name);
			local title = TooltipHandler:getTitle(name);
			local rpflag = TooltipHandler:getRPTooltipText(name, tooltipUnit);
			local cstatus = TooltipHandler:getCharStatusTooltipText(name, tooltipUnit);
			local friendstate = Friendlist:GetFriendstateText(name);

			--[[local guildText = "";
			if self:getGuildDisp() == 1 then
				local guild, a, b = GetGuildInfo(tooltipUnit);
				if guild ~= nil and guild ~= "" then
					guildText = guild;
				end
			end]]--
	 
			GameTooltipTextLeft1:SetText(name .. " " .. surname);
			GameTooltip:AddLine(title, 1, 1, 1);
			--[[if guildText ~= "" then
				GameTooltip:AddLine(guildText, 1, 1, 1);
			end]]--
			GameTooltip:AddLine(rpflag, 1, 1, 1);
			GameTooltip:AddLine(cstatus, 1, 1, 1);
			GameTooltip:AddLine(friendstate, 1, 1, 1);	 

			GameTooltip:Show();
		end
	end
end

--[[

FlagRSP:PostFlags()

-- Posts the player's current RP flags.

]]--

function FlagRSP:PostFlags()
	if ( self.isInitialized and not flagRSP_AFK and self.canPost) then
		if ( self:getRPFlag() ~= nil ) then 
			if ( self:getRPFlag() ~= 1 ) then
				ChatHandler:sendMessage("<RP" .. self:getRPFlag() .. ">");
			elseif ( self:getRPFlag() == 1 ) then 
				ChatHandler:sendMessage("<RP>");
			else
				ChatHandler:sendMessage("<RP0>");
			end
		else 
			ChatHandler:sendMessage("<RP0>");
		end
		
		if ( self:GetPlayerName() ~= nil and self:GetPlayerName() ~= UnitName("player") ) then
			local name = self:GetPlayerName();
			name = string.gsub(name, "<", "\\(");
			name = string.gsub(name, ">", "\\)");
			
			ChatHandler:sendMessage("<AN1>" .. name);
		else
			ChatHandler:sendMessage("<AN1>");
		end
		
		if ( self:getOwnSurname() ~= nil ) then 
			-- Empty names and titles should not be sent.
			local name = self:getOwnSurname();
			name = string.gsub(name, "<", "\\(");
			name = string.gsub(name, ">", "\\)");

			ChatHandler:sendMessage("<AN3>" .. name);
		else 
			ChatHandler:sendMessage("<AN3>");
		end
		
		if ( self:getOwnTitle() ~= nil ) then 
			-- Empty names and titles should not be sent.

			local title = self:getOwnTitle();
			title = string.gsub(title, "<", "\\(");
			title = string.gsub(title, ">", "\\)");

			ChatHandler:sendMessage("<T>" .. title);
		else
			ChatHandler:sendMessage("<T>");
		end
    
		--isStableVersion = false;
		if ( self.isStableVersion ) then
			local num = math.random(1,20);
			
			--self:printDebug("Random num for version posting is:" .. num);
			if ( num == 1 ) then
				local release = self.releaseVersion;
				local major = self.majorVersion;
				local minor = self.minorVersion;

				self:printDebug("Posting version number.");
				ChatHandler:sendMessage("<V>" .. release .. "." .. major .. "." .. minor);
			end
		end
    
		if ( FlagRSPConfigure:GetRevision() >= 0 and FlagRSPConfigure:GetDescription() ~= nil and FlagRSPConfigure:GetDescription() ~= "" ) then
			ChatHandler:sendMessage("<DV>" .. FlagRSPConfigure:GetRevision());
		elseif ( FlagRSPConfigure:GetRevision() >= 0 and FlagRSPConfigure:GetDescription() ~= nil and FlagRSPConfigure:GetDescription() == "" ) then
			ChatHandler:sendMessage("<DV>-1");
		end

		ChatHandler:sendMessage("<CS" .. self:getCharStatusNum() .. ">");
		
		self.canPost = false;
	end
end

function FlagRSP:getStats()
	local t = 0;
	local o = 0;
	local oMax = 0;

	--local flags = {};

	--for i = 0, 4 do
		--flags[i] = 0;
	--end

	local style;
	local lastSeen;
	local interval;
	for name, interv in pairs(flagRSP_IntervalList) do
		t = t + 1;
      
		--self:printDebug(name .. " is online?. Interval is: " .. interv);
		--local interval;
		--if flagRSP_IntervalList[name] ~= nil and flagRSP_LastSeenList[name] ~= nil then
		if flagRSP_LastSeenList[name] ~= nil then
			lastSeen = flagRSP_LastSeenList[name];
			interval = interv;

			--self:printDebug(name .. " is using old interval.");

			--if GetTime() <= interval*2+lastSeen then 
				--self:printDebug(name .. " is online.");
			--end
		end
		--self:printDebug("test " .. name .. " for new interval.");
		if flagRSP_IntervalListHigh[name] ~= nil and flagRSP_LastSeenListHigh[name] ~= nil then
			lastSeen = flagRSP_LastSeenListHigh[name];
			interval = flagRSP_IntervalListHigh[name];

			--self:printDebug(name .. " is using new interval.");

			--if GetTime() <= interval*2+lastSeen then 
				--self:printDebug(name .. "is online.");
			--end
		end
      
		if lastSeen ~= nil and interval ~= nil then
			if GetTime() <= interval*2+lastSeen then
				o = o + 1;
				oMax = oMax + 1;
			end
		else
			oMax = oMax + 1;
		end

		--style = FlagHandler:getFlag("RPFlag", name);

		--flags[style] = flags[style] + 1;

		--self:printDebug(style);
	end

	if o > self.maxUsersOnline then
		self.maxUsersOnline = o;
	end

	--FlagRSPSettings[self.rName][self.pName].numOnlineList[FlagRSPSettings[self.rName][self.pName].numOnlineList.position] = oMax;
	FlagRSPConfigure:SetOnlineListEntry(FlagRSPConfigure:GetOnlineListPosition(), oMax);
   
	return t, o, oMax;
end

function FlagRSP:plotStats()
	local t,o,oMax;

	self:printM("|CFFFFFF7FFLAGRSP STATISTICS:|r");
	self:printM("|CFFFFFF7FUsage statistics:|r");

	local flags = {};

	if GetTime() - self.sessionStartTick >= FLAGRSP_MAXINTERVALHIGH then
		t,o,oMax = self:getStats();
      
		self:printM("Currently |CFFFFFF7F" .. t .. "|r flagRSP users listed who were online during this session (these are all users who have sent flags during this session).");
		--self:printM("A maximum of |CFFFFFF7F" .. oMax .. "|r flagRSP users is currently online (Note: all pre-0.5.0 users are counted as online even if offline. Therefore this value is unreliable).");
		self:printM("Number of flagRSP users currently online: |CFFFFFF7F" .. o .. "|r (Note: only users of version 0.5.0 or above counted, reliable only if all users use post-0.5.0 versions).");
		self:printM("Maximum number of online users: |CFFFFFF7F" .. self.maxUsersOnline .. "|r (Note: only users of version 0.5.0 or above counted, reliable if all users use post-0.5.0 versions. This value shows how many users were active at the same time).");
		--self:printDebug("flagRSP: DEBUG: Total " .. t .. " players using flagRSP.");
	else
		self:printM("No usage statistics available yet. Please wait " .. math.ceil((FLAGRSP_MAXINTERVALHIGH - (GetTime() - self.sessionStartTick))/6)/10 .. " more minutes.");
	end

	self:printM("|CFFFFFF7FTraffic statistics:|r");
	self:printM("Current flag sending interval (low flags): |CFFFFFF7F" .. math.ceil(self.postInterval/6)/10 .. "|r minutes (" .. math.ceil(self.postInterval*10)/10 .. " seconds).");
	self:printM("Current flag sending interval (high flags): |CFFFFFF7F" .. math.ceil(self.postIntervalHigh/6)/10 .. "|r minutes (" .. math.ceil(self.postIntervalHigh*10)/10 .. " seconds).");
	if self.bytesPerSecondLastInterval ~= 0 then
		self:printM("Bytes per second in the last six minute interval: |CFFFFFF7F" .. math.ceil(self.bytesPerSecondLastInterval*10)/10 .. "|r");
		self:printM("Maximum bytes per second in one six minute interval: |CFFFFFF7F" .. math.ceil(self.bytesPerSecondIntervalMax*10)/10 .. "|r");
	end
	self:printM("Bytes per second (whole session): |CFFFFFF7F" .. math.ceil(self.bytesPerSession/(GetTime() - self.sessionStartTick)*10)/10 .. "|r");
	self:printM("Own description has been sent |CFFFFFF7F" .. self.ownDescriptionSent .. "|r times.");
	self:printM("Own description has been requested |CFFFFFF7F" .. self.ownDescriptionRequested .. "|r times.");

	self:printM("|CFFFFFF7FMemory statistics:|r");
	local memory, gcTreshold = gcinfo();

	self:printM("Memory used by complete interface (including addons): |CFFFFFF7F" .. math.ceil(memory*10)/10 .. "|r kB.");
	local numEntries, flags = FlagHandler:getStats();

	self:printM("Number of currently saved entries (i.e. characters) on all realms: |CFFFFFF7F" .. numEntries.. "|r.");

	self:printM("|CFFFFFF7FRP flags used:|r");
	self:printM("Number of flagRSP users without flag: |CFFFFFF7F" .. flags[0] .. "|r.");
	self:printM("Number of flagRSP users with standard flag: |CFFFFFF7F" .. flags[1] .. "|r.");
	self:printM("Number of flagRSP users with casual flag: |CFFFFFF7F" .. flags[2] .. "|r.");
	self:printM("Number of flagRSP users with fulltime flag: |CFFFFFF7F" .. flags[3] .. "|r.");
	self:printM("Number of flagRSP users with beginner flag: |CFFFFFF7F" .. flags[4] .. "|r.");

	if self.oldestVersion ~= "" or self.newestVersion ~= "" then
		self:printM("|CFFFFFF7FVersion usage:|r");
		if self.oldestVersion ~= "" then
			self:printM("Oldest detected version: |CFFFFFF7F" .. self.oldestVersion .. "|r");
		end
		if self.newestVersion ~= "" then
			self:printM("Newest detected version: |CFFFFFF7F" .. self.newestVersion .. "|r");
		end
	end
end

--[[

FlagRSP:OnUpdate(elapsed)

-- OnUpdate Handler

]]--

function FlagRSP:OnUpdate(elapsed)
	self:checkAFK("");
	FlagRSPConfigure:initializeSettings();
	
	-- Saves on calls to GetTime().
	local curTime = GetTime();
	
	if ( self.startTime == 0 ) then
		self.startTime = curTime;
	elseif ( not self.isInitialized and curTime > ( self.startTime + 30 ) ) then
		self.isInitialized = true;
		self:printM(FlagRSP_Locale_Initialized);
	end
	
	if ( curTime > self.newTickDisplayBox ) then
		FlagRSPInfo:displayBox("target");
		self.newTickDisplayBox = curTime + self.displayBoxInterval;
	end
	
	if ( self.displayOwnTooltip ) then
		self:showOwnTooltip();
	end
	
	if ( curTime > ( self.postFlagLast + self.postFlagInterval ) ) then
		self.canPost = true;
		self:PostFlags();
		self.postFlagLast = curTime;
	end
	
	if ( curTime > self.newTickChatFrame ) then
		self:muteDataChannel();
		self.newTickChatFrame = curTime + 60;
	end

	ChatHandler:executeQueue();

	if ( curTime > self.updateFriendsTick ) then
		TooltipHandler:updateFriendEntries();
		self.updateFriendsTick = curTime + 30;
	end

	if ( curTime > self.updateGuildListTick ) then
		TooltipHandler:updateGuildMembers();
		self.updateGuildListTick = curTime + 300;
	end

	if ( curTime > self.updateIgnoredListTick ) then
		TooltipHandler:updateIgnoredList();
		self.updateIgnoredListTick = curTime + 30;
	end

	-- flokru: Targetframe updates on several events. I don't know which.
	--         So, we change level in targetframe here. Not an optimal solution.
	if ( FlagRSPConfigure:isInitialized() and FlagRSPConfigure:GetLevelDisplay() == "relative" ) then
		if UnitName("target") ~= nil then
			local targetLevel = UnitLevel("target");
			local targetLevelText = "";
			if targetLevel > 0 then
				local Diff = targetLevel - UnitLevel("player");
	    
				if UnitClassification("target") == "elite" then
					Diff = Diff + 7;
				end
	    
				if Diff <= -7 then targetLevelText = FlagRSP_Locale_TF_Epuny; end
				if Diff <= -5 and Diff > -7 then targetLevelText = FlagRSP_Locale_TF_Puny; end
				if Diff < -1 and Diff > -5 then targetLevelText = FlagRSP_Locale_TF_Weak; end
				if Diff >= -1 and Diff <= 1 then targetLevelText = FlagRSP_Locale_TF_Equal; end
				if Diff > 1 and Diff < 4 then targetLevelText = FlagRSP_Locale_TF_Strong; end
				if Diff >= 4 and Diff < 7 then targetLevelText = FlagRSP_Locale_TF_Vstrong; end
				if Diff >= 7 and Diff < 10 then targetLevelText = FlagRSP_Locale_TF_Estrong; end
				if Diff >= 10 then targetLevelText = FlagRSP_Locale_TF_Impossible; end
				if targetLevel == -1 then targetLevelText = FlagRSP_Locale_TF_Impossible; end

				TargetLevelText:SetText(targetLevelText);
			end
		end
	end
end


function FlagRSP:sendRPTicket(msg)
	self:JoinChannel();
	local id = GetChannelName(xTP_ChannelName);

	if msg ~= nil and msg ~= "" then
		SendChatMessage("<RPT>" .. "[" .. GetRealZoneText() .. "]" .. msg, "CHANNEL", FlagRSP_Locale_CLanguage, id); 
		--ChatHandler:sendMessage("<RPT>" .. "[" .. GetRealZoneText() .. "]" .. msg, "CHANNEL");
		self:printM("|cffff0000" .. FlagRSP_Locale_RPTicket .. "|r " .. msg);
	end
end


function FlagRSP:isAFKEvent()
	local msg = arg1;
	if ( string.find(msg,FlagRSP_Locale_AFK) ) then
		return true;
	end
	return false;
end

function FlagRSP:isnotAFKEvent()
	local msg = arg1;
	if (string.find(msg,FlagRSP_Locale_NOTAFK)) then
		return true;
	end
	return false;
end


--[[

FlagRSP:chatMessageHandler(arg1, arg2)

-- new handler for incoming chat messages from data channel.

]]--
function FlagRSP:chatMessageHandler(arg1, arg2)
	if string.find(arg1,"<") == nil then 
		return;
	end

	if string.find(string.sub(arg1,2,string.len(arg1)),"<") ~= nil then 
		-- more than one option in line.
      
		--self:printDebug("Multioption line: " .. arg1);
      
		local firstBrack = string.find(string.sub(arg1,2,string.len(arg1)),"<");

		local firstComm = string.sub(arg1,1,firstBrack);
		local rest = string.sub(arg1,firstBrack+1, string.len(arg1));

		--self:printDebug("found first command: " .. firstComm .. ", passing rest");
      
		self:chatMessageHandler(firstComm, arg2);
		self:chatMessageHandler(rest, arg2);
   
	elseif string.find(string.sub(arg1,2,string.len(arg1)),"<") == nil then 
		-- just one command. treat it as usual.

		--self:printDebug("Single line: " .. arg1);

		--self:printDebug("before: " .. arg1);

		-- substitute encoded brackets.
		arg1 = string.gsub(arg1, "\\%(", "<");
		arg1 = string.gsub(arg1, "\\%)", ">");

		--self:printDebug("after: " .. arg1);

		if string.sub(arg1,1,3) == "<RP" and string.sub(arg1,4,4) ~= "T"  then
			--local Found = 0;
			local RPStyle = string.sub(arg1,4,4);

			if RPStyle == "4" then
				FlagHandler:addFlag("RPFlag", 4, arg2);
			elseif RPStyle == "3" then
				FlagHandler:addFlag("RPFlag", 3, arg2);
			elseif RPStyle == "2" then
				FlagHandler:addFlag("RPFlag", 2, arg2);
			elseif RPStyle == "0" then
				FlagHandler:addFlag("RPFlag", 0, arg2);
			else
				FlagHandler:addFlag("RPFlag", 1, arg2);
			end
	 
			--self:printDebug(arg1);
			--self:printDebug(arg2);

			local int;
			if (string.len(arg1) >= 6 and RPStyle ~= ">") then
				int = tonumber(string.sub(arg1,6,string.len(arg1)));
			elseif (string.len(arg1) >= 5 and RPStyle == ">") then
				int = tonumber(string.sub(arg1,5,string.len(arg1)));
			end
		 
			if int ~= nil and int > 0 then
				flagRSP_IntervalList[arg2] = int;
				flagRSP_LastSeenList[arg2] = GetTime();
				--self:printDebug("we see that " .. arg2 .. " is using interval: " .. flagRSP_IntervalList[arg2] .. " and now it is: " .. flagRSP_LastSeenList[arg2]);
			end
		elseif string.sub(arg1, 1, 7) == "<TITEL>" then
			FlagHandler:addFlag("Title", string.sub(arg1, 8, string.len(arg1)), arg2);
		elseif string.sub(arg1, 1, 3) == "<T>" then
			FlagHandler:addFlag("Title", string.sub(arg1, 4, string.len(arg1)), arg2);
		elseif string.sub(arg1, 1, 6) == "<NAME>" then
			FlagHandler:addFlag("Surname", string.sub(arg1, 7, string.len(arg1)), arg2);
		elseif string.sub(arg1, 1, 3) == "<N>" then
			FlagHandler:addFlag("Surname", string.sub(arg1, 4, string.len(arg1)), arg2);
		elseif string.sub(arg1, 1, 9) == "<CSTATUS>" then
			flagRSP_CharStatusList[arg2] = string.sub(arg1, 10, string.len(arg1));
			--FlagHandler:addFlag("CStatus", string.sub(arg1, 10, string.len(arg1)), arg2);
		elseif string.sub(arg1, 1, 3) == "<CS" then
			local cStatus = string.sub(arg1,4,4);
			local csString = "";
			
			if cStatus == "0" then
				csString = "";
			elseif cStatus == "1" then
				csString = "ooc";
			elseif cStatus == "2" then
				csString = "ic";
			elseif cStatus == "3" then
				csString = "ffa-ic";
			elseif cStatus == "4" then
				csString = "st"; 
			end
			flagRSP_CharStatusList[arg2] = csString;
			--FlagHandler:addFlag("CStatus", csString, arg2);
			
			local int;
			if string.len(arg1) >= 6 then
				int = tonumber(string.sub(arg1,6,string.len(arg1)));
			end
		 
			if int ~= nil and int > 0 then
				flagRSP_IntervalListHigh[arg2] = int;
				flagRSP_IntervalList[arg2] = int;
				flagRSP_LastSeenListHigh[arg2] = GetTime();
				flagRSP_LastSeenList[arg2] = GetTime();

				--self:printDebug("we see that " .. arg2 .. " is using interval: " .. flagRSP_IntervalListHigh[arg2] .. " and now it is: " .. flagRSP_LastSeenListHigh[arg2]);
			end
		 
		elseif string.sub(arg1, 1, 6) == "<DREV>" then
			local n = tonumber(string.sub(arg1, 7, string.len(arg1)));

			if n == nil then n=-1; end
			if n == -1 then
				FlagRSP_CharDesc.savedRev[arg2] = -1;
			end

			FlagRSP_CharDesc.availRev[arg2] = n;
			FlagRSP_CharDesc.savedRev[arg2] = FlagHandler:getDescRev("A", arg2);

			if FlagRSP_CharDesc.savedRev[arg2] ~= FlagRSP_CharDesc.availRev[arg2] then
				if FlagRSP_CharDesc[arg2] ~= nil then
					for i=1, table.getn(FlagRSP_CharDesc[arg2]) do
						FlagRSP_CharDesc[arg2][i] = nil;
					end
				end
				FlagRSP_CharDesc.savedRev[arg2] = -1;

				FlagHandler:delDesc("A", arg2);
			end
		 
			if FlagRSP_CharDesc.savedRev[arg2] ~= FlagRSP_CharDesc.availRev[arg2] and FlagRSP_CharDesc.availRev[arg2] ~= -1 then
				local num = self:getTrafficDependendRandom(9, 0.025);
				--math.random(1,20);
				--self:printDebug("Random num for version posting is:" .. num);
				if num == 0 then
					self:printDebug("pulling desc for: " .. arg2);
					FlagRSPInfo:pullDescription(arg2);
				end
			end

			--self:printDebug("flagRSP: DEBUG: Player " .. arg2 .. " has a description with revision: " .. FlagRSP_CharDesc.availRev[arg2]);

		elseif string.sub(arg1, 1, 4) == "<DV>" then
			local n = tonumber(string.sub(arg1, 5, string.len(arg1)));
		 
			if n == nil then n=-1; end
			if n == -1 then
				FlagHandler:SetRevision(arg2, n);
			end
		 
--~ 			FlagRSP_CharDesc.availRev[arg2] = n;
--~ 			FlagRSP_CharDesc.savedRev[arg2] = FlagHandler:getDescRev("A", arg2);

			-- If the current revision doesn't match the old one,
			-- delete the current description, set the new revision,
			-- and pull a new description.
			if ( n ~= FlagHandler:GetRevision(arg2) and n ~= -1 ) then
				FlagHandler:delDesc("A", arg2);
				FlagHandler:SetRevision(arg2, n);
				FlagRSPInfo:pullDescription(arg2);
			end

			--self:printDebug("flagRSP: DEBUG: Player " .. arg2 .. " has a description with revision: " .. FlagRSP_CharDesc.availRev[arg2]);
		
		elseif ( string.sub(arg1, 1, 3) == "<AN" ) then
			-- Support for FlagRSP2's Additional Name tags
			local n = tonumber(string.sub(arg1, 4, 4));
			
			if ( n == 1 ) then
				-- Character's first name.
				if ( arg1 == "<AN" .. n .. ">" ) then
					-- If it's an empty tag, clear the stored flag.
					FlagHandler:addFlag("Name", "", arg2);
				else
					FlagHandler:addFlag("Name", string.sub(arg1, 6, string.len(arg1)), arg2);
				end
			elseif ( n == 2 ) then
				-- Character's middle name.
				if ( arg1 == "<AN" .. n .. ">" ) then
					FlagHandler:addFlag("Middle", "", arg2);
				else
					FlagHandler:addFlag("Middle", string.sub(arg1, 6, string.len(arg1)), arg2);
				end
			elseif ( n == 3 ) then
				-- Character's last name.
				if ( arg1 == "<AN" .. n .. ">" ) then
					FlagHandler:addFlag("Surname", "", arg2);
				else
					FlagHandler:addFlag("Surname", string.sub(arg1, 6, string.len(arg1)), arg2);
				end
			end
		elseif arg1 == ("<DPULL>" .. self.pName) then
			FlagRSPInfo:PushDescription();
		elseif arg1 == ("<DP>" .. self.pName) then
			FlagRSPInfo:PushDescription();
		elseif string.sub(arg1, 1, 6) == "<DVER>" then
			self:checkVersion(string.sub(arg1, 7, string.len(arg1)));
		elseif string.sub(arg1, 1, 3) == "<V>" then
			self:checkVersion(string.sub(arg1, 4, string.len(arg1)));
		elseif string.sub(arg1, 1, 2) == "<D" and tonumber(string.sub(arg1,3,4)) ~= nil then
			FlagRSPInfo:saveDescription(string.sub(arg1,3,4), string.sub(arg1,6,string.len(arg1)), arg2);
		elseif string.sub(arg1, 1, 2) == "<P" and tonumber(string.sub(arg1,3,4)) ~= nil then
			FlagRSPInfo:saveDescription(string.sub(arg1,3,4), string.sub(arg1,6,string.len(arg1)), arg2, true);
		end
		Friendlist:updateEntry(arg2, "flagRSP");
	end
end


--[[

FlagRSP:getTrafficDependendRandom(q, k)

-- Returns a number between 0 and x with x = q*exp[(traffic-targetTraffic)*k].

]]--
function FlagRSP:getTrafficDependendRandom(q, k)
	local traFlagRSP_Locale_NextTipButtonffic, targetTraffic;
	local traffic;

	if self.bytesPerSecondLastInterval ~= 0 then
		traffic = self.bytesPerSecondLastInterval;
	else
		traffic = self.bytesPerSession/(GetTime() - self.sessionStartTick);
	end

	targetTraffic = self.postingTargetBps + self.postingTargetBpsHigh;

	local upperLimit = math.ceil(q*math.exp(k*(traffic-targetTraffic)));

	self:printDebug("traffic-targetTraffic is: " .. traffic-targetTraffic);
	self:printDebug("upperLimit is: " .. upperLimit);

	if upperLimit > 1000 then
		upperLimit = 1000;
	end

	if upperLimit == 0 then
		return 0;
	else
		return math.random(0, upperLimit);
	end
end